from evdspy.EVDSlocal.common.files import *
from dataclasses import dataclass, field

from datetime import datetime


@dataclass
class Report:
    file_name: str = "stats_report.txt"

    @property
    def date_content(self):
        now = datetime.now()

        current_time = now.strftime("%Y-%m-%d %H:%M:%S")
        return current_time

    def add(self, content):
        content = f"{self.date_content} => {content}"
        WriteAdd(self.file_name, content)


def report_test():
    r = Report(file_name="test_stats_report.txt")
    r.add("content")


from ..messages.error_classes import *
from evdspy.EVDSlocal.helper.print_m import *
from evdspy.EVDSlocal.initial_setup.setup_folders import check_folders_setup_necessary

if not check_folders_setup_necessary():
    print("setup checked...")
    try:

        from .load_commands_ import *

        # help_evds()
        # setup_now()
        # help_evds()

    except OptionsFileNotLoadedError:
        print("load_commands_ error trying setup")
        time.sleep(3)
        from ..initial_setup.initial_setups import *
        SetupInitial().setup()


else:
    from .load_commands_ import *

    # help_evds()



from evdspy.EVDSlocal.requests_.lower_energy import apikey_works


def test_apikey_worksM():
    assert apikey_works("s1") is True, "s1 is not True"
    assert apikey_works("f1") is False, "s1 is not True"



import evdspy
from evdspy.EVDSlocal.main_ import *

def test_load_general():
    assert callable(setup_series), "setup_series is not callable"
    assert callable(help_), "help_ is not callable"


def test_setup_now():
    # evdspy.setup_now()
    # evdspy.help_()
    # evdspy.check()
    easy_setup()
    get()


from evdspy.EVDSlocal.config.apikey_class import ApikeyClass


def test_set_apikey():
    assert ApikeyClass().set_api_key_filetype(value="test") == "test", "set api key did not work"
    assert ApikeyClass().set_api_key_runtime(value="testruntime") == "testruntime", "set api key did not work"


def test_get_apikey():
    assert ApikeyClass().set_api_key_filetype(value="test") == "test", "get api key did not work"
    assert ApikeyClass().get_api_key_fromfile() == "test", "get api key did not work"
    assert ApikeyClass().set_api_key_filetype("test3") == "test3", "set api key did not work"
    assert ApikeyClass().get_api_key_fromfile() == "test3", "get api3 key did not work"
    assert ApikeyClass().get_api_key_runtime() == "testruntime", "get testruntime  key did not work"

def test_valid_key():
    assert ApikeyClass().set_api_key_filetype(value="thatisfromFile") == "thatisfromFile", "get api key did not work"
    ApikeyClass().now_testing_is_key_is_valid = "thisisValid"
    # assert ApikeyClass().get_valid_api_key() == "thisisValid", "get thisisValid  key did not work"

from ..initial.load_commands_ import *

# def test_create_series_file():
#     assert evdspy.create_series_file() is True, "test_create_series_file"
#

from ..initial.start_options import *

#
# def test_check_if_this_is_pytest():
#     import sys
#     print(sys.argv)
#
#     assert evdspy.check_if_this_is_pytest() is True, f"Could not detect pytest is running if running {sys.argv}"
#

"""
    TODO 
    series.txt not found error check 
    create file test
    apikey not working error test
    proxy test
    graphs
    summary
        
"""



# ------------------------------------------------------
#
#       main_.py
#                           package: evdspy @2022
# ------------------------------------------------------
import time

from .initial.start_args import *
from evdspy.EVDSlocal.initial_setup.setup_folders import check_folders_setup_necessary
from evdspy.EVDSlocal.initial_setup.initial_setups import SetupInitial
from evdspy.EVDSlocal.common.files import Read

test_api_key_set = False
test_args = test_args

from pathlib import Path

try:
    import version__
except:
    pass


def get_develop_vers_main():
    print(Path.cwd())
    parent = Path(__file__).parent
    v = Read(Path(parent / "version__.py")) # EVDSlocal/ version.py
    return v


print(f"version:{get_develop_vers_main()}")
time.sleep(2)

deb("....setting up...")

# ---------------------------------SetupInitial----setup ---------


if not check_folders_setup_necessary():
    SetupInitial().setup()
    # SetupInitial().create_series_text_ex()

# ---------------------------------SetupInitial----setup ---------


from .initial.try_loading_initial_commands import *

from .initial.load_commands_ import *

assert callable(check)
assert callable(get)
assert callable(menu)

menu_onload()
EV = help_

# menu()
# help_()



import functools
import random
import time

# --------------------------------------------------------------------------------------
if __name__ != "__main__":
    from evdspy.EVDSlocal.common.files import WriteBytes, ReadBytes, WriteBytesAdd
    from evdspy.EVDSlocal.config.config import ConfigBase, config
    from evdspy.EVDSlocal.manual_requests.prepare import basic_for_test, PrepareUrl
else:
    def WriteBytes(file_name: str, content_bytes: bytes):
        with open(file_name, 'wb+') as f:
            f.write(content_bytes)


    def WriteBytesAdd(file_name: str, content_bytes: bytes):
        with open(file_name, 'ab') as f:
            f.write(content_bytes)


    def ReadBytes(file_name: str):
        with open(file_name, "rb") as f:
            return f.read()
# --------------------------------------------------------------------------------------
import base64
import hashlib
import os
from pathlib import Path
from dataclasses import dataclass
from typing import List, Tuple
from typing import Union, Callable

from evdspy.EVDSlocal.initial.start_options import default_arch_folder_name , default_arch_file_name
if config.current_mode_is_test:
    default_arch_folder_name = f"EVDSlocal/{default_arch_folder_name}"

item_sep = "\n"
item_sep_bytes = bytes(item_sep, encoding="utf8")


def get_pepper():
    """ trying to get extra sec by getting info that will be change for all users
        but will not change very frequently.
        If it changes it will just result a new request until it changes
        again though, so not a big deal.
    """
    pepper = "The quick brown fox, why not"
    import os
    try:
        u = os.environ.get("USERNAME")
    except:
        u = "any_user"
    try:
        p = Path()
    except:
        p = "path"
    pepper = f"{pepper}-{u}-{p}"
    return pepper


@dataclass
class ArchFile():
    path: Union[str, Path] = ""
    file_name: Union[str, Path] = default_arch_file_name

    def __post_init__(self):
        if isinstance(self.path, str):
            self.path = Path.cwd() / default_arch_folder_name
        self.file_name = self.path / default_arch_file_name


arch = ArchFile()


def create_folder_if_not_exist():
    if not arch.path.is_dir():
        os.makedirs(arch.path)


def setup_is_ok():
    return ArchFile().file_name.is_file()


def one_way(x, num=27):
    x = x.encode("utf-8")
    hasher = hashlib.sha1(x)
    return base64.urlsafe_b64encode(hasher.digest()[:num])


def with_pepper(x: str):
    pepper = get_pepper()
    return f"{pepper}{x}"


def name_format(api_key: str, status_code: Union[bool, int]) -> str:
    status_code = convert_status_code(status_code)
    return f"{api_key}-{get_pepper()}-{status_code}"


shuffle = True


def shuffle_content(old_content: bytes, bytes_api_key_pepper) -> bytes:
    old_list: List[bytes] = old_content.split(item_sep_bytes)
    new_list: List[bytes] = old_list + [bytes_api_key_pepper]
    content: bytes = functools.reduce(lambda x, p: x + p + item_sep_bytes, new_list)
    return content


def add_checked_hashes(api_key: str, status_code: Union[bool, int]):
    status_code: int = convert_status_code(status_code)
    create_folder_if_not_exist()
    bytes_api_key_pepper = one_way(name_format(api_key, status_code))
    assert isinstance(status_code, bool) is False, "something w"
    if shuffle:
        if not setup_is_ok():
            old_content: bytes = bytes("start", encoding="utf-8")
            new_content = bytes_api_key_pepper + item_sep_bytes
        else:
            old_content: bytes = ReadBytes(str(arch.file_name))
            new_content: bytes = shuffle_content(old_content, bytes_api_key_pepper + item_sep_bytes)
        WriteBytes(str(arch.file_name), new_content)
    else:
        WriteBytesAdd(str(arch.file_name), bytes_api_key_pepper + item_sep_bytes)


def get_list_of_cache():
    content_byte: bytes = ReadBytes(arch.file_name)
    item_list: List[bytes] = content_byte.split(item_sep_bytes)
    item_list_str: Tuple[str] = tuple(map(str, item_list))
    return item_list_str


def convert_status_code(status_code_new: Union[bool, int]) -> int:
    if isinstance(status_code_new, int) and not isinstance(status_code_new, bool):
        return status_code_new
    obj = {True: 200, False: 500}
    status_code_new = obj[status_code_new]
    return status_code_new


def check_api_key_hashes_opt(api_key, status_code: Union[bool, int]):
    status_code: int = convert_status_code(status_code)
    new_hash = one_way(name_format(api_key, status_code))
    return str(new_hash) in get_list_of_cache()


def populate_first_rand():
    if not setup_is_ok():
        create_folder_if_not_exist()
    for item in range(100):
        status_code = 500  # random.choice((200, 500))
        x_tuple = (f"test-{item}", status_code)
        # add_checked_hashes(f"test-{item}", status_code)
        add_checked_hashes(*x_tuple)


@dataclass
class Checked:
    never_checked: bool
    checked_before_and_passed: bool
    checked_before_and_failed: bool


def check_if_api_key_was_checked_before(api_key: str) -> Checked:
    """
    Every time user gives an api key to save
        we need to check if it is working.
        In earlier versions we were checking with
        a new request each time without caching.
        This version this will be a better solution which  attempts to optimize
        min request and min collision of one way hashing.

            Functions of this file are only for minimizing new request on setting api key.
            @params
                api_key : str
    """
    checked_before_and_passed = check_api_key_hashes_opt(api_key, 200)
    checked_before_and_failed = check_api_key_hashes_opt(api_key, 500)
    never_checked = not any((checked_before_and_passed, checked_before_and_failed))
    return Checked(never_checked, checked_before_and_passed, checked_before_and_failed)


testing = False
test_apikeys_success = ("s1", "s2", "s3", "s4", 'testApiKeyMock')
test_apikeys_fails = ("f1", "f2")
common_test_words = ("test", "t", "api", "key", "apikey", "key", "tested")

from ..common.colors import *

def apikey_works(api_key, check_func_request: Callable = basic_for_test, testing_=False):
    result = apikey_works_helper(**locals())
    msg_success = "Your api key was tested with success."
    msg_fails = "Your api key was tested with success."
    if result:
        print_with_success_style(msg_success)
    else:
        print_with_failure_style(msg_fails)
    return result


def apikey_works_helper(api_key, check_func_request: Callable = basic_for_test, testing_=False):
    if not setup_is_ok():
        populate_first_rand()
    ans: Checked = check_if_api_key_was_checked_before(api_key)
    checked_before = not ans.never_checked
    if checked_before:
        if ans.checked_before_and_passed:
            return True
        if ans.checked_before_and_failed:
            return False
        raise "something wrong lower energy"
    # check request
    """ make request """
    if testing or \
            testing_ or \
            config.current_mode_is_test or \
            api_key in test_apikeys_success or \
            api_key in test_apikeys_fails or \
            api_key in common_test_words:
        """ for testing """
        status_code_new: bool = api_key in test_apikeys_success  # random.choice((200, 500,))
        print_with_creating_style("first part ", api_key, status_code_new)

    else:
        if api_key in common_test_words:
            print_with_failure_style(f"api key does not look like a key. Returning...{api_key}")
            return apikey_works_helper(api_key, testing_=True)
            # raise "cannot make a new request"
        # temp test forced
        # print("temp test forced")
        # return apikey_works_helper(api_key, testing_=True)
        print_with_updating_style("WARNING making a new request to test your Api key ")
        time.sleep(2)
        status_code_new: bool = check_func_request(api_key)
    """ add one way comp. hash to archive """
    add_checked_hashes(api_key, status_code_new)
    return status_code_new


__all__ = [
        'apikey_works'
]



from ..common.common_imports import *

TrueFunc = (lambda x: True)


def folder_creatable(folder):
    try:
        d = Path(folder)
        if d.is_dir():
            return True
    except:
        return False
    return False


def folder_creatable_by_adding(folder):
    try:
        d = Path(folder)
        e = str(d).replace(d.stem, "")
        if not len(e) > 0:
            return False
        f = Path(e)
    except:
        return False
    return folder_creatable(f)


from evdspy.EVDSlocal.common.folder_name_checks import is_relative

from rich import print
from rich.console import Console
from rich.theme import Theme

custom_theme = Theme({
    "info": "dim cyan",
    "warning": "magenta",
    "danger": "bold red"
})
console = Console(theme=custom_theme)


# console.print("test info", style="info")
# console.print("[warning]test warning[/warning]")
# console.print("Test", style="danger")


def folder_format_check(folder: str):
    if not folder.strip():
        return True
        # print(f"checking ... {folder}")
    console.print(f"checked foldername :  {folder}", style="warning")
    import string
    if isinstance(folder, str) and is_relative(folder):
        return True
    print("[danger]folder name does not fit to format[/danger]")
    return False


def notest():
    t = is_relative(r"/asfasdf/asdfasdf")
    is_relative(r"C:\Users\ppdsppe\PycharmProjects\evdspy\evdspy\SeriesData")
    is_relative(r"C:/asfasdf/asdfasdf??")
    is_relative(r"C:/asfasdf/asdfasdf??")
    is_relative(r"asfasdf/asdfasdf")


notest()



from typing import Callable, List, Union, Tuple
from dataclasses import dataclass, field
from abc import ABC, abstractmethod
from enum import Enum
from typing import Callable, Protocol, List, Tuple, Union
from datetime import datetime

default_api_domain = "https://evds2.tcmb.gov.tr/service/evds/"


@dataclass
class ApiDomain(ABC):
    domain: str


@dataclass
class EVDSApiDomain(ApiDomain):
    domain: str = default_api_domain

    def __repr__(self):
        return self.domain


# --------------------------------Frequency---------------------
class FrequencyEnum(Enum):
    daily = 1
    business_day = 2
    weekly = 3
    semimonthly = 4
    monthly = 5
    quarterly = 6
    semi_annual = 7
    annual = 8
    default = 5


class AggregationEnum(Enum):
    average = "avg"
    min = "min"
    max = "max"
    first = "first"
    last = "last"
    cumulative = "sum"
    default = "avg"


class FormulasEnum(Enum):
    level = 0
    percentage_change = 1
    difference = 2
    yoy_percentage_change = 3
    yoy_difference = 4
    per_cha_com_end_of_pre_yea = 5
    dif_com_end_of_pre_yea = 6
    moving_average = 7
    moving_sum = 8
    default = 0


from typing import Callable


def get_enum_with_value(key: Union[str, int], enum_: Union[str, int, Callable], default_value=None) -> Union[str, int]:
    """ typical usages
        get_enum_with_value( 5 , Frequency , Frequency.Monthly )

    """
    if isinstance(key, int):
        key = str(key)

    if hasattr(default_value, "value"):
        """otherwise probably None """
        default_value = getattr(default_value, "value", None)

    dd = {str(x.value): x for x in enum_}
    enum_value: Enum = dd.get(key, None)

    if hasattr(enum_value, "value"):
        v = getattr(enum_value, "value")
        return v
    if not default_value:
        return getattr(default_value, "default")
    return default_value


@dataclass
class UrlParam(ABC):
    """UrlParam class """
    value: Union[str, datetime, int, Tuple[str], Callable, Enum, List[str]]
    initial: str = "anything"
    number_of_repeat: int = 1
    disp_value: str = ""
    required: bool = True
    obscured: bool = False

    def __str__(self) -> str:
        return f"{self.disp_value}"

    def __repr__(self) -> str:
        return f"{self.disp_value}"

    def __post_init__(self):
        self.create_disp_value()

    def create_disp_value(self):
        if not self.value:
            self.disp_value = f"{self.initial}=Noneapi.params.103"
            return

        # if isinstance(self.value, Enum) :
        if hasattr(self.value, "value"):
            self.disp_value = f"{self.initial}={self.value.value}"
        if isinstance(self.value, (tuple, list,)):
            value: str = "-".join(self.value)
            self.disp_value = f"{self.initial}={value}"
            return
        if isinstance(self.value, (str, int,)):
            self.disp_value = f"{self.initial}={self.value}"
            return
        if self.value is None:
            if self.required:
                raise f"{self.__class__.__name__} parameter is required"
            return
        self.disp_value = f"{self.initial}={self.value}"

    def reportable_value(self):
        if self.obscured:
            return f"{self.initial}=obscured_for_report"
        return self.disp_value

    def explanation(self):
        if self.obscured:
            return f"{self.initial}: obscured_for_report"
        if callable(self.value):
            disp_value = f"{self.initial}: {self.value.__name__}"
            return disp_value
        if isinstance(self.value, (list, str,)):
            disp_value = f"{self.initial}: {', '.join(self.value)}"
            return disp_value
        return f"{self.initial}: {self.value}"


@dataclass
class Apikey(UrlParam):
    initial: str = "key"
    obscured: bool = True


@dataclass
class Series(UrlParam):
    """ Frequency """
    initial: str = "series"
    value: Union[List[str], Tuple[str]] = ()


@dataclass
class Frequency(UrlParam):
    """ Frequency """
    # value: Union[str, datetime, int, Tuple, Callable, Enum]
    initial: str = "frequency"
    required: bool = False


@dataclass
class Formulas(UrlParam):
    """ Formulas """
    initial: str = "formulas"
    required: bool = False

    def create_disp_value(self):
        repeat = "-".join([str(self.value) for x in range(self.number_of_repeat)])
        self.disp_value = f"{self.initial}={repeat}"


@dataclass
class Aggregations(UrlParam):
    """ Aggregations """
    initial: str = "aggregationTypes"
    required: bool = False

    def create_disp_value(self):
        repeat = "-".join([str(self.value) for x in range(self.number_of_repeat)])
        self.disp_value = f"{self.initial}={repeat}"


@dataclass
class DateStart(UrlParam):
    """ DateStart """
    initial: str = "startDate"


@dataclass
class DateEnd(UrlParam):
    """ DateEnd """
    value: str = ""
    initial: str = "endDate"


def convert_list_params_to_str(alist: Union[List[UrlParam], Tuple[UrlParam]], fnc: Union[Callable, str] = str):
    separator_char = "&"
    if fnc == str:
        alist = [str(x) for x in alist if x is not None]
        alist_str: str = separator_char.join(alist)
    else:
        indent = " " * 10
        block = "-" * 25
        separator_char = "\n" + indent
        if fnc == "report":
            alist = [x.reportable_value() for x in alist if x.reportable_value() is not None]
        if fnc == "explanation":
            alist = [x.explanation() for x in alist if x.explanation() is not None]
        alist_str: str = block + f"{fnc}" + block + "\n" + indent + separator_char.join(alist)
    return alist_str



from evdspy.EVDSlocal.utils.utils_general import *
from evdspy.EVDSlocal.series_format.series_format_config_funcs import *

mainSepBegin = "---Series---------------------------------"
mainSepEnd = "---/Series---------------------------------"
GSEP = "--++--"
NEW_LINE = "\n"
example_codes = ['TP.EVDSSTATS.KAYITLI', 'TP.EVDSSTATS.ZIYARET']
example_codes_str = ",".join(example_codes)

# ----------------------------- S E R I E S ---------------------------------------------------
items_from_user_series = [
        "Data folder",
        "Subject",
        "Name prefix(Optional) Default : `EVPY_` ",
        "EVDS data series codes",
        "Frequency",
        "Formulas",
        "AggregateType",
]

default_prefix = "EVPY_"
freq = f"""
Daily: 1
Business: 2
Weekly(Friday): 3
Semimonthly: 4
Monthly: 5
Quarterly: 6
Semiannual: 7
Annual: 8
"""
frequency_dict = {
        1: 'Daily',
        2: 'Business day',
        3: 'Weekly(Friday)',
        4: 'Semimonthly',
        5: 'Monthly',
        6: 'Quarterly',
        7: 'Semiannual',
        8: 'Annual',
}
default_freq = "5"
formulas_ = f"""
Level: 0
Percentage change: 1
Difference: 2
Year-to-year Percent Change: 3
Year-to-year Differences: 4
Percentage Change Compared to End-of-Previous Year: 5
Difference Compared to End-of-Previous Year : 6
Moving Average: 7
Moving Sum: 8
"""
formulas_dict = {
        0: "Level",
        1: "Percentage change",
        2: "Difference",
        3: "Year-to-year Percent Change",
        4: "Year-to-year Differences",
        5: "Percentage Change Compared to End-of-Previous Year",
        6: "Difference Compared to End-of-Previous Year",
        7: "Moving Average",
        8: "Moving Sum",
}
default_formulas_ = "0"
AggregateType_ = f"""
Average: avg,
Minimum: min,
Maximum: max
Beginning: first,
End: last,
Cumulative: sum
"""
quickNoteForParams = f"""
Frequencies
-----------------
{freq}
`Formulas`s
-----------------
{formulas_}
Aggregate types
-----------------
{AggregateType_}
"""
AggregateType__valid_answers = ["avg", "min", "max", "first", "last", "sum"]
default_AggregateType_ = "avg"
explanations_series = [
        rf"Path for your project {NEW_LINE}{indent}(Example: output or C:\Users\Userx\MyTempData ) default: {default_prefix}Data "
        rf"{NEW_LINE} This should be an empty folder. If folder does not exist"
        "program will create a new folder with this name",
        rf"Subject will be used to name excel files. {NEW_LINE}{indent}(Example: Visitors, Tourism, Balance of Payments, Exchange Rates etc.",
        "If there is already excel files in the folder this will prevent replacing existing files.  ",
        rf"evdspy will create your series.txt file {NEW_LINE} it will add data series codes if you provide. "
        rf"{NEW_LINE} {indent} example : {example_codes_str} {NEW_LINE}"
        f"You may also continue with default values and modify the config_series.cfg file later if you prefer",
        f"{freq}\nFrequency (Number between 1, 8  e.g.(monthly) : 5) default: 1",
        f"{formulas_}\n e.g. (Number between 0, 8  e.g.(Level) : 0) default: 0 ",
        f"{AggregateType_}\n e.g. ( avg / min / max / first / last / sum  ) default: avg",
]


def check_valid_answer(items, answer: str):
    return answer in items


def check_valid_answer_freq(answer: str):
    return check_valid_answer(tuple(map(str, range(1, 8))), answer) or answer.strip() == ""


def check_valid_answer_formulas_(answer: str):
    return check_valid_answer(tuple(map(str, range(0, 9))), answer) or answer.strip() == ""


def check_valid_answer_aggr(answer: str):
    return check_valid_answer(AggregateType__valid_answers, answer) or answer.strip() == ""


default_answer_subject = 'SubjectNotGiven'
default_answers_series = [f"{default_prefix}Data",
                          default_answer_subject,
                          default_prefix,
                          example_codes_str,
                          default_freq,
                          default_formulas_,
                          default_AggregateType_]
check_funcs_series = [folder_format_check,
                      TrueFunc,
                      TrueFunc,
                      TrueFunc,
                      TrueFunc,
                      TrueFunc,
                      TrueFunc]
# import string
# x = "aa,ab-ac"
# s = x.split()
#
""" Transform Functions """
same = lambda x: x
trim_string = lambda x: x.strip()
from typing import Tuple

# import re
# split_items = lambda x: re.split("[,-/\n;]+", x)
split_items = lambda text: tuple(text.translate(text.maketrans({x: "-" for x in "[,-/\n;]~"})).split("-"))
""" Default answers if user go with '' Empty answer """


def get_default_for_question(answer, question):
    obj = {
            "folder": f"{default_prefix}Data",
            "prefix": default_prefix,
            "freq": default_freq,
            "formulas": default_formulas_,
            "aggr": default_AggregateType_,
            "series": example_codes,
            "subject": default_answer_subject
    }
    if not answer.strip():
        return obj.get(question, answer)
    return answer.strip()


# instead of functools partial lambda this time
bound_question = lambda question: lambda answer: get_default_for_question(answer, question)
funcs_names = ("folder", "subject", "prefix", "series", "freq", "formulas", "aggr",)
transform_answers_series: Tuple = tuple(map(lambda x: bound_question(x), funcs_names))



from .colors import *

filename = 't.txt'
content = 'aaa'
from ..common.common_imports import *
from typing import Union
from pathlib import Path


class WriteContext:
    def __init__(self, fname, content, msg, mode="w"):
        self.mode = mode
        self.fname = fname
        self.content = content
        self.msg = msg

    def __enter__(self):
        self.file_ = open(self.fname, self.mode, encoding='utf-8')
        self.file_.write(self.content)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.file_.close()
        if exc_tb:
            print_with_failure_style(exc_tb)
            print_with_failure_style(self.msg)
        return True


def Write(fname="test.txt", content="test", msg="Error"):
    info = f"{fname}"

    result = False, "File was not created"
    with WriteContext(fname, content, msg):
        result = True, f"File was created... : {info}"
    return result, f"File was created.. : {info}"


def WriteAdd(fname="test.txt", content="test", msg="Error"):
    info = f"{fname}"
    content = "\n" + content

    result = False, "File was not created"
    with WriteContext(fname, content, msg, mode="a"):
        result = True, f"File was created... : {info}"
    return result, f"File was created.. : {info}"


class ReadContext:
    def __init__(self, fname, msg):
        self.fname = fname
        self.msg = msg

    def __enter__(self):
        # self.file_ = open(self.fname, "r", encoding='utf-8')
        self.file_ = open(self.fname, "r", encoding='utf_8_sig')
        return self.file_

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.file_.close()
        if exc_tb:
            print_with_failure_style(exc_tb)
            print_with_failure_style(self.msg)
            return False
        return True


def Read(file_name: Union[str, Path], msg="Error"):
    if isinstance(file_name, str):
        file_name = Path(file_name)
    content = ""
    with ReadContext(file_name, msg) as file_:
        content = file_.read()
    return content


def create_folder(folder_name: Union[str, Path]):
    if isinstance(folder_name, str):
        folder_name = Path(folder_name)
    try:
        os.makedirs(folder_name)
    except FileExistsError:
        print_with_updating_style("folder {} already exists".format(folder_name))
    except Exception as e:
        print_with_failure_style(e)


def is_file(path_: str):
    d = Path(path_)
    return d.is_file()


def is_dir(path_: str):
    d = Path(path_)
    return d.is_dir()


def WriteBytes(file_name: str, content_bytes: bytes):
    with open(file_name, 'wb') as f:
        f.write(content_bytes)


def WriteBytesAdd(file_name: str, content_bytes: bytes):
    with open(file_name, 'ab') as f:
        f.write(content_bytes)


def ReadBytes(file_name: str):
    with open(file_name, "rb") as f:
        return f.read()


def make_indented(content: str, indent=" " * 10) -> str:
    content_tup: tuple = tuple(f"{indent}{x}" for x in content.splitlines())
    content = "\n".join(content_tup)
    return content


def add_one_line(cont_tuple: tuple, line: str) -> tuple:
    cont: list = list(cont_tuple)
    new_cont = cont + [line]
    return tuple(new_cont)


# def content_continues_line(content: str) -> str:
#     content = make_indented(content)
#     content = make_it_summary(content, add_continue=True)
#     return content
def line_continues_fn(skipped=0):
    def cont_fn():
        lns = "lines"
        if skipped == 1:
            lns = "line"
        if skipped > 0:
            msg = f"{skipped} {lns} more..."
            # skipped_msg = "" if skipped > 0 else f"{skipped} line more..."
            cont = "." * 15 + f"{msg}" + "." * 15
            print_with_style(f"{cont}\n")
        ...

    return cont_fn


def make_it_summary(content: str, max_line_num: int = 10, add_continue: bool = False) -> tuple:
    content_tup: tuple = tuple(f"{x}" for x in content.splitlines())
    new_content_tup: tuple = content_tup[0:max_line_num]
    skipped_lines = len(content_tup) - max_line_num if len(content_tup) - max_line_num > 0 else 0
    new_content_tup = add_one_line(new_content_tup, "." * 15)
    return "\n".join(new_content_tup), skipped_lines


def file_exists_show_and_return(file_name: Union[str, Path]) -> bool:
    if isinstance(file_name, str):
        file_name = Path(file_name)
    if file_name.is_file():
        content_ = Read(file_name)
        print_with_updating_style(f"file {file_name} exists, not replacing...\n")
        print_with_color("content: \n -------\n", "blue")
        cont_msg, skipped = make_it_summary(make_indented(content_), add_continue=False, max_line_num=10)
        print(cont_msg)
        """print fn will return """
        line_continues_fn(skipped)()
        time.sleep(1)
        return True
    return False



from .help_messages import *
from .cmd_formats import *

api_key_not_set_msg = f"""
        ---------------------------------------------------------------        
               WARNING
        ---------------------------------------------------------------
        Api key not set . 
        you may continue with saving your api key by answering 
                 `yes` 
        Program will store your api key in a private area and read it when 
        it needs to request data from EVDS server. 
        You may get your api key from your EVDS account on EVDS website.
        
        Note : Alternatively you may continue with providing a text file 
        address that contains your api key. 
        (Please choose `provide api key file` option for this option.)
    
"""
you_need_to_run_setup_options = f"""
    it looks like options.py not created.
    try command below from command line in your project's root folder.

{setup_now_command}
        

"""

copying_options_msg = "Found an options file... Trying to copy it to temporary env."

series_file_not_found_error_msg = f""" 
create_series_file

"""



from pathlib import Path
import os
from evdspy.EVDSlocal.common.files import Write, Read
import base64

api_key_folder_name = Path() / "APIKEY_FOLDER"
api_key_file_name = api_key_folder_name / "api_key.txt"


def decode(base64_bytes) -> str:
    if isinstance(base64_bytes, str):
        base64_bytes = str.encode(base64_bytes)
    str_bytes = base64.b64decode(base64_bytes)
    decoded = str_bytes.decode("ascii")
    return decoded


def encode(text: str) -> bytes:
    t_bytes = text.encode("ascii")
    encoded = base64.b64encode(t_bytes)
    return encoded


def WriteBytes(file_name: str, content_bytes: bytes):
    with open(file_name, 'wb') as f:
        f.write(content_bytes)


def ReadBytes(file_name: str):
    with open(file_name, "rb") as f:
        return f.read()


from evdspy.EVDSlocal.config.apikey_class import ApikeyClass


# def check_key_before_saving(instance, api_key_answer: str):
#     ApikeyClass().now_testing_is_key_is_valid = api_key_answer
#     return instance.check_api_key_with_simple_Wrapper()


def save_api_key_to_file(instance , api_key_answer ):
    instance.wait(1)

    if not api_key_folder_name.is_dir():
        os.makedirs(api_key_folder_name)
    # api_key_answer = instance.get_input("Your api key : ")
    # Write(api_key_file_name, str(encode(api_key_answer)), "")

    WriteBytes(str(api_key_file_name), encode(api_key_answer))
    # test for file api key for runtime (temp)
    ApikeyClass().set_api_key_filetype(value=api_key_answer)

        # ApikeyClass().now_testing_is_key_is_valid = api_key_answer
    print("your api file was saved...")
    instance.wait(1)
    # instance.set_apikey(api_key_answer)
    api_key_from_file  = get_api_key_from_file_improved()
    return api_key_from_file


def get_api_key_from_file_improved():
    if not api_key_file_name.is_file():
        return False

    def read_api_file():
        content = ReadBytes(str(api_key_file_name))
        return content

    pot_key = read_api_file()
    if pot_key:
        return decode(pot_key)
    return False


def check_api_key_on_load():
    if api_key := get_api_key_from_file_improved():
        ApikeyClass().set_api_key_filetype(value=api_key)



from ..common.common_imports import *
from pathlib import Path


def check_if_this_is_pytest():
    def check():
        path_ = Path(sys.argv[0])
        return "pytest" in str(path_.stem)

    if len(sys.argv) > 0 and check():
        return True
    else:

        return False



import sys

from ..components.evds_files import *
from ..components.evds_seri_files import test_series_
from ..utils.utils_general import *
from ..initial.start_args import Args
from ..components.options_class import Options, load_options
from ..config.apikey_class import *

#from ...main import *

options_ = load_options()
args = Args(sys.argv)
test_evds = EvdsSorgu(
    options_=options_,
    session=None,
    series_=test_series_,
    file_name='pytest_req',
    args=args
)
test_evds_series = EvdsSorguSeries(
    options_,
    session=None,
    series_=test_series_,
    file_name='pytest_req',
    args=args
)

"""

TODO
    
    => 
    
"""


def test_create_class():
    assert test_evds is not None, "test_evds not created"


# def test_evds_save_excel():
#     assert test_evds.kaydet_excel() is True, "test_evds not created"
#
#
# def test_test_evds_series_save_excel():
#     assert test_evds_series.kaydet_excel() is True, "test_evds not created"
#
#
# def test_test_evds_series_save_excel_returns_smt():
#     assert test_evds_series.kaydet_excel() in [True, False], "test_evds not created"
#
folder = get_current_dir()


# r"C:\Users\ppdsppe\PycharmProjects\EVDS.3.0\svde2\src\EVDSlocal\tests\\"


def test_get_proxy_from_file():
    assert get_proxy_from_file(
        folder / ".." / "tests" / r"test.proxy.txt") == "http://proxy.example.com", "Proxy test failed"


def test_get_api_key():
    assert get_api_key_from_file(folder / ".." / "tests" / r"test.api.txt") == "testApiKey", "Api key test failed"


def test_global_var_api_key():
    assert isinstance(global_var_api_key(), str)



from evdspy.EVDSlocal.series_format.series_creator import *

from ..tests.core_options_test import *


# def test_SeriesFileFormat(capsys):
#     with capsys.disabled():
#         # print(test_SeriesFileFormat2())
#         ...
#     # SeriesFileFormat
#
#     assert isinstance(test_SeriesFileFormat2()[0],
#                       EvdsSeriesRequestWrapper), "SeriesFileFormat did not get EvdsSeriesRequest Item "
#
#     with capsys.disabled():
#         if verbose:
#             create_series_file_from_Wrapper(test_SeriesFileFormat2(), 'test3.txt')
#         ...


from evdspy.EVDSlocal.series_format.read_config_series import *
from evdspy.EVDSlocal.series_format.populate_series import check_pop_read


# def test_read_series_config2(capsys):
#     read_series_config()
#     with capsys.disabled():
#         if verbose:
#             print(check_pop_read())
#
#
# def test_clean_series_line(capsys):
#     with capsys.disabled():
#         print("*************test_clean_series_line TEST*************")
#         print(PopulateSeries().split_series_file())

    # req = PopulateSeries().split_series_file()

    # print(req )
    # def check_pop_read():
    #     ps = PopulateSeries()
    #     req = ps.split_series_file(content=pop_series_test_content)
    #     deb2(req)
    #     return req

#
# def test_create_locked_series_file(capsys):
#     from evdspy.EVDSlocal.initial.load_modules import LoadModulesClass
#     with capsys.disabled():
#         print("*************test_clean_series_line TEST*************")
#         print(LoadModulesClass().create_locked_series_file())



from evdspy.EVDSlocal.utils.utils_general import *


class Session:

    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, 'instance'):
            cls.instance = super(Session, cls).__new__(cls)
            cls.start(cls)

        return cls.instance

    def start(cls):
        cls.hash = get_random_hash(5)

    def attach_logger(cls, logger_):
        cls.logger = logger_

    def __repr__(self):
        return f"session:{self.hash}"

    # def __new__(cls, sObj = None , DB = None , *args, **kwargs):
    #     if not hasattr(cls, 'instance'):
    #         cls.db = DB(cls)
    #         cls.annoying = False
    #         if sObj is not None :
    #             cls.start(cls, sObj)
    #         cls.instance = super(Session, cls).__new__(cls)


def get_session():
    return Session()


def remove_api_key(url):
    if "key=" not in url:
        return url
    separator = "&"
    new_parts = (x for x in url.split(separator) if "key=" not in x)
    return separator.join(new_parts) + "&key=xxxOMITTEDforSec"



from ..common.common_imports import *

# from ..initial.try_loading_initial_commands import *

def examples_get(get_function ):
    """ It will try to make request using default api key and proxy if set """
    get_function()



from evdspy.EVDSlocal.setup_project.user_setup_helper import *

from evdspy.EVDSlocal.components.populate_series import *
from evdspy.EVDSlocal.logs.log_template import *


def read_series_config():
    req = PopulateSeries("config_series.cfg").split_series_file()
    print(req)
    deb(req)



from ..common.common_imports import *
from dataclasses import dataclass
from typing import Union
from enum import Enum, auto


class ApiKeyType(Enum):
    runtime = auto()
    from_file = auto()
    from_file_options = auto()


@dataclass
class ApiKeyValue:
    name: str
    value: Union[str, bool] = False
    type_: ApiKeyType = ApiKeyType.runtime


@dataclass
class ApiKeyDict:
    runtime_apikey: ApiKeyValue = ApiKeyValue("runtime", False, ApiKeyType.runtime)
    from_file_apikey: ApiKeyValue = ApiKeyValue("fromfile", False, ApiKeyType.from_file)
    from_file_options_api_key: ApiKeyValue = ApiKeyValue("fromfile_options", False, ApiKeyType.from_file_options)

    def set_value(self, type_: Union[str, ApiKeyType], value):
        if type_ in ["runtime", ApiKeyType.runtime]:
            self.runtime_apikey.value = value
            return
        if type_ in ["fromfile", ApiKeyType.from_file]:
            self.from_file_apikey.value = value
            return
        self.from_file_options_api_key = value




class ApikeyClass(object):
    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(ApikeyClass, cls).__new__(cls)
            cls.post_init(cls)
        return cls.instance

    def __repr__(cls):
        return f"""
   cls.now_testing_is_key_is_valid: {cls.now_testing_is_key_is_valid:}
   cls.current_key_using : {cls.current_key_using}
   cls.APIKEYDict : {cls.APIKEYDict}
"""

    def post_init(cls):

        cls.APIKEYDict: ApiKeyDict = ApiKeyDict()
        cls.now_testing_is_key_is_valid: Union[str, bool] = False
        cls.current_key_using: Union[str, bool] = cls.APIKEYDict.runtime_apikey.value

    def get_valid_api_key(cls):

        if cls.now_testing_is_key_is_valid:
            return cls.now_testing_is_key_is_valid

        key_objs = cls.get_api_keys()
        if not key_objs:
            return False
        return key_objs[0].value

    def set_api_key_runtime(cls, value: str):
        cls.instance.APIKEYDict.set_value(ApiKeyType.runtime, value)
        return cls.instance.APIKEYDict.runtime_apikey.value

    def set_api_key_filetype(cls, value: str):
        cls.instance.APIKEYDict.set_value(ApiKeyType.from_file, value)

        return cls.instance.APIKEYDict.from_file_apikey.value

    def set_api(cls, key: str):
        print("set api not implemented")
        raise "set api not implemented"

    def get_api_keys(cls):
        keys = [
            cls.instance.APIKEYDict.runtime_apikey,
            cls.instance.APIKEYDict.from_file_apikey,
            cls.instance.APIKEYDict.from_file_options_api_key,
        ]
        return [x for x in keys if x.value is not False]

    def get_api_keys_dict(cls):
        return cls.instance.APIKEYDict

    def get_api_key_runtime(cls):
        return cls.instance.APIKEYDict.runtime_apikey.value

    def get_api_key_fromfile(cls):
        return cls.instance.APIKEYDict.from_file_apikey.value

    def get_api_key_fromfile_options(cls):
        return cls.instance.APIKEYDict.from_file_options_api_key.value



# -------------------------------------------------------------------------
# from evdspy.EVDSlocal.before_main_ import *
from evdspy.EVDSlocal.components.evds_files import EvdsSorguSeries
from evdspy.EVDSlocal.components.evds_seri_files import EvdsSeri, EvdsSeriesRequest, EvdsSeriesRequestWrapper
from evdspy.EVDSlocal.series_format.populate_series import PopulateSeries
from evdspy.EVDSlocal.series_format.series_creator import SeriesFileFormat, create_series_file_from_Wrapper, \
    create_series_text_file
from evdspy.EVDSlocal.initial.start_args import Args
from evdspy.EVDSlocal.components.options_class import *
from evdspy.EVDSlocal.components.evds_seri_files import test_series_
from ..initial_setup.setup_folders import check_setup
from ..messages.error_classes import *
from evdspy.EVDSlocal.log_classes.log_template import deb
from evdspy.EVDSlocal.config.config import ConfigBase

from evdspy.EVDSlocal.manual_requests.prepare import basic_for_test, PrepareUrl

from evdspy.EVDSlocal.config.apikey_class import ApikeyClass


@dataclass
class LoadModulesClass():
    options_: Optional[any] = load_options()
    args: Optional[Args] = Args(sys.argv)
    evds_list: field(default_factory=list) = ()
    series_filename: str = default_series_file_name
    api_key_test: bool = False
    # TODO
    """
    """

    def placebo_item(self):
        evds = EvdsSorguSeries(
                options_=self.options_,
                session=None,
                subject="placebo",
                series_=test_series_,
                file_name="placebo",
                args=self.args)
        return evds

    def summary(self):
        if not self.evds_list or not len(self.evds_list) > 0:
            print("No EvdsSorguSeries yet")
            return
        for item in self.evds_list:
            item.summary()

    def series_from_file_test(self):
        """Reads series' code from given file name inside initial folder default :`series.txt` """
        return self.create_EVDSSorguSeries()

    def main(self):
        return self.series_from_file()

    def series_from_file(self):
        """ MAIN function """
        result = []
        try:
            result = self.series_from_file_core()
        except SeriesFileDoesNotExists:
            print(SeriesFileDoesNotExists().message)

        return result

    def convert_wrapperlist_to_text(self, wrapper_list: List[EvdsSeriesRequestWrapper]):
        content = create_series_file_from_Wrapper(wrapper_list)
        return content

    def create_locked_series_file(self):
        wrapper_list = self.populate_evds_wrapperList()
        content = self.convert_wrapperlist_to_text(wrapper_list)
        file_name = default_series_file_name.split(".cfg")[0] + "-locked.cfg"
        create_series_text_file(file_name, content)

    def create_EVDSSorguSeries(self):
        return self.populate_evds_list()

    def populate_evds_wrapperList(self):
        # return
        # if self.api_key_test:
        #     """ api key already was tested here we are checking if it is valid """
        #     self.series_filename = str(
        #         ConfigBase().runtime_file_name_root_path / "series_format" / "series_test_runtime.cfg")
        #     # self.series_filename = "series_format/series_test_runtime.cfg"
        sff = SeriesFileFormat(self.series_filename)
        return sff.EvdsSeriesRequestWrapperList

    def check_for_apikey_with_simple_wrapper(self, api_key):

        basic_for_test(api_key)
        exit()

        self.api_key_test = True
        self.evds_list = self.populate_evds_list()
        if not self.evds_list:
            return False
        self.evds_list = [self.evds_list[0]]
        return self.evds_to_excel(api_key_test=True)

        # return len(evds_list) and isinstance(evds_list[0], EvdsSorguSeries)

    def populate_evds_list(self):
        """==>>>>   (Deep )   PopulateSeries() =>
                    (Surface) SeriesFileFormat .EvdsSeriesRequestWrapper_
        Reads series' code from given file name
        inside initial folder default :`series.txt`
        returns a list of wrapper
        ( EvdsSeriesRequestWrapper1 ,  EvdsSeriesRequestWrapper2 , ... )
        EvdsSeriesRequestWrapper :
            name
            subject
            EvdsSeriesRequest
        """
        list_ = []
        EvdsSeriesRequestWrapperList = self.populate_evds_wrapperList()
        for index, EvdsSeriesRequestWrapper_ in enumerate(EvdsSeriesRequestWrapperList):
            evds = EvdsSorguSeries(
                    options_=self.options_,
                    session=None,
                    subject=EvdsSeriesRequestWrapper_.subject,
                    series_=EvdsSeriesRequestWrapper_.EvdsSeriesRequest_,
                    file_name=EvdsSeriesRequestWrapper_.name,
                    args=self.args,
                    bfs=EvdsSeriesRequestWrapper_.bfs,
            )

            list_.append(evds)
        return list_

    def evds_to_excel(self, api_key_test=False):
        for evds in self.evds_list:
            try:

                evds.save_excel()
                evds.report()
            except Exception as exc:
                # print(exc)
                # print(evds)
                deb(str(exc))

        return True

    def series_from_file_core(self):
        self.evds_list = self.create_EVDSSorguSeries()
        self.evds_to_excel()
        self.create_locked_series_file()
        return True

    def seri_ornek(self, options_, args):
        liste = [
                'TP.IHRACATBEC.9999',
                'TP.IHRACATBEC.31',
                'TP.IHRACATBEC.41',
        ]
        seri_evds_objs: list[EvdsSeri] = [EvdsSeri(x) for x in liste]
        series_ = EvdsSeriesRequest(self.options_, seri_evds_objs)
        file_name = "test34"
        evds = EvdsSorguSeries(
                session=None,
                series_=series_,
                file_name=file_name
        )
        evds.save_excel(evds.file_name)

    def check(self):
        evds_items = self.series_from_file_test()
        if len(evds_items):
            self.check_items(evds_items)

    def check_items(self, evds_items):
        config = ConfigBase()
        options_copy_file_name = config.runtime_file_name_path
        user_options_file_name = config.user_options_file_name
        deb(config)
        # args = Args(sys.argv)
        ps = PopulateSeries()
        series_file_was_created = False
        f = Path(ps.input_file_name)
        if f.is_file():
            series_file_was_created = True

        folders_ok = False
        if check_setup():
            folders_ok = True
        api_key_is_ok = False
        if ApikeyClass().get_valid_api_key():
            api_key_is_ok = True

        def display():

            options_display = SingletonOptions().check()

            hiddenok = f"ok(hidden)"
            if not api_key_is_ok:
                hiddenok = ""
            msg = f"""
        Checking installation and other setup requirements :
        ----------------------------------------------------
        Folders created          :      {folders_ok}
        Series file was created  :      {series_file_was_created} {indent} {ps.input_file_name}
        Api key was set          :      {api_key_is_ok} {indent} {hiddenok}
        {options_display}
                """
            print(msg)

        display()



from ..common.common_imports import *
from ..utils.utils_general import *
from ..initial.start_options import *


@dataclass
class ConfigBase(ABC):
    cancel_request_temp: bool = False
    current_mode_is_test: bool = check_if_this_is_pytest()
    projectName: str = 'evdspy'
    interiorFolder = 'EVDSlocal'
    runtime_file_name_root_path = get_current_dir() / '..'
    runtime_file_name_path = get_current_dir() / '..' / "components" / "options.py"
    user_options_file_name = Path.cwd() / 'IO' / 'options.py'
    user_options_file_name_locked = Path.cwd() / 'IO' / 'options-locked.py'


config = ConfigBase()



from dataclasses import dataclass, field
from collections import namedtuple
import numpy
import requests
import sys
import pandas as pd
from pathlib import Path
from typing import Optional
from rich import print, inspect
from typing import List, Dict, Optional

import os
from pathlib import Path
from subprocess import call
import os
import time
from abc import ABC, abstractmethod
from typing import Optional
from typing import TypeVar
import base64
from datetime import date, datetime
from enum import Enum
import functools

from evdspy.EVDSlocal.helper.print_m import *

from evdspy.EVDSlocal.log_classes.log_template import deb, deb2, debug




#   ----------------------------------------------------------      ExcelSaveClass
from dataclasses import dataclass
import pandas as pd

from evdspy.EVDSlocal.common.colors import print_with_creating_style, print_with_failure_style, \
    print_excel_created_style
from evdspy.EVDSlocal.utils.utils_general import *
from typing import List
from evdspy.EVDSlocal.common.common_imports import *
@dataclass
class ExcelSaveClass():
    df: pd.DataFrame
    file_name: str
    excel_out_ext: str = "xlsx"
    def out_name_format(self, name):
        return f"{name}.{self.excel_out_ext}"
    def save_with_name(self):
        self.current_file_created = self.out_name_format(self.file_name)
        self.df.to_excel(self.current_file_created)
        return True
    def save_with_diff_name(self):
        h = get_random_hash(5)
        self.current_file_created = self.out_name_format(f"{self.file_name}-{h}")
        self.df.to_excel(self.current_file_created)
        return True
    def try_in_order(self, f_list: List[callable]):
        for index, f in enumerate(f_list):
            result = self.try_this(f)
            if result:
                return index, result
        return -1, False
    def try_this(self, f):
        try:
            f()
            return True
        except Exception as exc:
            print(exc)
            return False
    def save_excel__(self):
        index, result = self.try_in_order(
            [
                self.save_with_name,
                self.save_with_diff_name
            ]
        )
        if index > -1:
            print_excel_created_style(f"\n{indent}{self.current_file_created} was created...")
            return True
        print_with_failure_style(f"File  {self.current_file_created} not created...\n\n\n")
        return False
#   ----------------------------------------------------------    / ExcelSaveClass



from ..common.common_imports import *
from collections import namedtuple
from dataclasses import dataclass, field
from ..config.config import *
from evdspy.EVDSlocal.console.screen import *
from ..common.prog import rich_sim
from ..common.colors import *

# from evdspy.utils.utils_general import *

indent = " " * 15

# MenuItem = namedtuple("MenuItem", "func disp sleeptime clear")
# MenuItem = namedtuple("MenuItem", "func disp sleeptime clear")
from typing import Union, List, Tuple, Callable

do_nothing = lambda: True


@dataclass
class MenuItem:
    func: Union[Callable, str]
    disp: str
    sleeptime: int = 0
    clear: bool = False
    name: str = ""
    callable_func: Callable = do_nothing

    def __post_init__(self):
        if callable(self.func):
            self.name = self.func.__name__
            self.callable_func = self.func
        elif isinstance(self.func, str):
            self.name = self.func
        else:
            self.name = self.disp

        self.get_sleep_times()
        self.get_screen_clear_bool()

    def get_sleep_times(self):
        obj = {"check": 1, "setup": 2, "get": 2}
        self.sleeptime = obj.get(self.name, 1)

    def get_screen_clear_bool(self):
        obj = {"check": False, "get": False, "help_": False}
        self.clear = obj.get(self.name, True)


@dataclass
class MenuMaker:
    menu_items: field(default_factory=List[MenuItem])
    # menu_items_display : field(default_factory=list)
    message: str = "\n" + "." * 25 + f"Selection ?"
    exit_item: bool = True
    exit_ = False
    exit_menu_call_back: Callable = do_nothing

    def __post_init__(self):
        self.add_exit_item()

    def exit(self):
        self.exit_ = True
        self.exit_menu_call_back()
        return
        # self.display()

    def title(self):

        print("\n" * 2)
        print_with_info_style("-" * 50)
        print_with_info_style(" " * 20 + " M E N U " + " " * 21)
        print_with_info_style("-" * 50)

    def add_exit_item(self):
        if self.exit_item:
            exit_menu_item = MenuItem(self.exit, "console")
            self.menu_items = tuple(list(self.menu_items) + [exit_menu_item])

    def display(self):

        if self.exit_:
            return
        self.title()
        for index, item in enumerate(self.menu_items):
            dec = f"{indent}{index + 1}. {item.disp} "
            print_menu_item_style(dec)
        self.get_choice()

    def menu_exists_and_callable(self, a: str):
        if str(a).isnumeric() and int(a) in (x for x in range(1, len(self.menu_items) + 1)):
            menu_item: MenuItem = self.menu_items[int(a) - 1]
            func = menu_item.func
            return callable(func), menu_item.callable_func, menu_item
        return False, False, False

    def get_choice(self):
        if self.exit_:
            return
        if config.current_mode_is_test:
            return
        ans = input(self.message)
        cond, func, menu_item = self.menu_exists_and_callable(ans)

        if cond:
            wait_seconds = menu_item.sleeptime

            if menu_item.clear:
                Screen().clear()
            v = func()
            if v == -1:
                self.exit_ = True
            # print("done...returning to menu...")
            rich_sim(wait_seconds, "completing")
            # time.sleep(wait_seconds)
        else:
            Screen().clear()
        return self.display()


__all__ = [
        'MenuItem',
        'MenuMaker',
]




sample_series = f"""
file:visitors
subject:visitors
TP.ODEMGZS.BDTTOPLAM
TP.ODEMGZS.ABD
TP.ODEMGZS.ARJANTIN
TP.ODEMGZS.BREZILYA
TP.ODEMGZS.KANADA
TP.ODEMGZS.KOLOMBIYA
TP.ODEMGZS.MEKSIKA
TP.ODEMGZS.SILI
TP.ODEMGZS.VENEZUELLA
TP.ODEMGZS.DAMERIKA
TP.ODEMGZS.AMERIKATOP
TP.ODEMGZS.CEZAYIR
TP.ODEMGZS.FAS
TP.ODEMGZS.GAFRIKA
TP.ODEMGZS.LIBYA
TP.ODEMGZS.MISIR
TP.ODEMGZS.SUDAN
TP.ODEMGZS.TUNUS
TP.ODEMGZS.DAFRIKA
TP.ODEMGZS.AFRIKATOP
TP.ODEMGZS.BAE
TP.ODEMGZS.BAHREYN
TP.ODEMGZS.BANGLADES
TP.ODEMGZS.CIN
TP.ODEMGZS.ENDONEZYA
TP.ODEMGZS.FILIPINLER
TP.ODEMGZS.GKORE
TP.ODEMGZS.HINDISTAN
TP.ODEMGZS.IRAK
TP.ODEMGZS.IRAN
TP.ODEMGZS.ISRAIL
TP.ODEMGZS.JAPONYA
TP.ODEMGZS.KKTC
TP.ODEMGZS.KATAR
TP.ODEMGZS.KUVEYT
TP.ODEMGZS.LUBNAN
TP.ODEMGZS.MALEZYA
TP.ODEMGZS.PAKISTAN
TP.ODEMGZS.SINGAPUR
TP.ODEMGZS.SURIYE
TP.ODEMGZS.SUUDARABISTAN
TP.ODEMGZS.TAYLAND
TP.ODEMGZS.URDUN
TP.ODEMGZS.YEMEN
TP.ODEMGZS.DASYA
TP.ODEMGZS.ASYATOP
TP.ODEMGZS.AVUSTRALYA
TP.ODEMGZS.YENIZELLANDA
TP.ODEMGZS.OKYANUSYA
TP.ODEMGZS.MILLIYESIZ
TP.ODEMGZS.GTOPLAM
--
file:visitors2
subject:visitors2
TP.ODEMGZS.BDTTOPLAM
TP.ODEMGZS.ABD
TP.ODEMGZS.ARJANTIN
TP.ODEMGZS.BREZILYA
TP.ODEMGZS.KANADA
TP.ODEMGZS.KOLOMBIYA
TP.ODEMGZS.MEKSIKA
TP.ODEMGZS.SILI
TP.ODEMGZS.VENEZUELLA
TP.ODEMGZS.DAMERIKA
TP.ODEMGZS.AMERIKATOP
TP.ODEMGZS.CEZAYIR
TP.ODEMGZS.FAS
TP.ODEMGZS.GAFRIKA
TP.ODEMGZS.LIBYA
TP.ODEMGZS.MISIR
TP.ODEMGZS.SUDAN
TP.ODEMGZS.TUNUS
TP.ODEMGZS.DAFRIKA
TP.ODEMGZS.AFRIKATOP
TP.ODEMGZS.BAE
TP.ODEMGZS.BAHREYN
TP.ODEMGZS.BANGLADES
TP.ODEMGZS.CIN
TP.ODEMGZS.ENDONEZYA
TP.ODEMGZS.FILIPINLER
TP.ODEMGZS.GKORE
TP.ODEMGZS.HINDISTAN
TP.ODEMGZS.IRAK
TP.ODEMGZS.IRAN
TP.ODEMGZS.ISRAIL
TP.ODEMGZS.JAPONYA
TP.ODEMGZS.KKTC
TP.ODEMGZS.KATAR
TP.ODEMGZS.KUVEYT
TP.ODEMGZS.LUBNAN
TP.ODEMGZS.MALEZYA
TP.ODEMGZS.PAKISTAN
TP.ODEMGZS.SINGAPUR
TP.ODEMGZS.SURIYE
TP.ODEMGZS.SUUDARABISTAN
TP.ODEMGZS.TAYLAND
TP.ODEMGZS.URDUN
TP.ODEMGZS.YEMEN
TP.ODEMGZS.DASYA
TP.ODEMGZS.ASYATOP
TP.ODEMGZS.AVUSTRALYA
TP.ODEMGZS.YENIZELLANDA
TP.ODEMGZS.OKYANUSYA
TP.ODEMGZS.MILLIYESIZ
TP.ODEMGZS.GTOPLAM
"""



# from ..common.common_imports import *
import time
from dataclasses import dataclass
from collections import namedtuple
from typing import Callable, List, Tuple, Union
from ..common.colors import *

from evdspy.EVDSlocal.components.options_class import SingletonOptions
from evdspy.EVDSlocal.config.config import *
from evdspy.EVDSlocal.common.files import *
from evdspy.EVDSlocal.setup_project.user_series_config import *
from evdspy.EVDSlocal.setup_project.user_series_config import formulas_dict, frequency_dict
from evdspy.EVDSlocal.setup_project.user_setup_helper import series_title
from evdspy.EVDSlocal.setup_project.user_options_config import items_from_user_config, \
    explanations_config, default_answers_config, check_funcs_options, transform_answers_options

from evdspy.EVDSlocal.config.config import config
from evdspy.EVDSlocal.initial.start_options import default_cache, default_end_date, default_start_date

# ----------------------------------------------------SetupInputs---------------------------

# -----------------------------------------------------------------------------------------
# SetupInputsConfig = namedtuple('SetupInputsConfig', 'cache_freq gl_date_start gl_date_end')
# SetupInputsSeries = namedtuple('SetupInputsSeries',
#                                'data_folder subject prefix ecodes frequency formulas aggregateType')

cache_freq = SingletonOptions().options_.default_cache
gl_date_start = SingletonOptions().options_.default_start_date
gl_date_end = SingletonOptions().options_.default_end_date


@dataclass
class SetupInputsConfig:
    cache_freq: str = SingletonOptions().options_.default_cache
    gl_date_start: str = SingletonOptions().options_.default_start_date
    gl_date_end: str = SingletonOptions().options_.default_end_date


@dataclass
class SetupInputsSeries:
    data_folder: str
    subject: str
    prefix: str
    ecodes: tuple
    frequency: str
    formulas: str
    aggregateType: str


# -----------------------------------------------------------------------------------------


from enum import Enum


# ----------------------------------------------------configType---------------------------
class configType(Enum):
    Series = 1
    Config = 2


# ----------------------------------------------------InputItem----------------------

@dataclass
class InputItem:
    msg: str
    long_msg: str = ""
    check_func: Callable = TrueFunc
    transform_func: Callable = same
    optional: bool = True

    # -----------------------------------------------------------------------------------------


"""
            GetInputFromUser
"""

# -----------------------------------------------------------------------------------------
from abc import ABC, abstractmethod


# ----------------------------------------------------GetInputFromUser----------------------
@dataclass
class GetInputFromUser(ABC):
    items: List[InputItem]
    config_type: configType

    @abstractmethod
    def ask_user(self, msg, check_func):
        pass

    @abstractmethod
    def get_from_user(self, item, explanation, check_func):
        pass

    @abstractmethod
    def process(self):
        pass


@dataclass
class GetInputFromUserBase():
    items: List[InputItem]
    config_type: configType

    def ask_user(self, msg, check_func):
        ans = input(msg)
        if not check_func(ans):
            return self.ask_user(msg, check_func)
        return ans

    def get_from_user(self, item, explanation, check_func):
        window = f"*" * 50 + "\n"
        msg = f"{window}**{item} \n {explanation} :"
        if config.current_mode_is_test:
            return 'pytest running'
        # ans = input(msg)
        ans = self.ask_user(msg, check_func)
        if not ans.strip():
            msg = "==> Continued with default value ..."
            print_with_updating_style(msg)
        print_with_creating_style(f"{window}")
        return str(ans)

    def process(self):
        assert config.current_mode_is_test is False, "config.current_mode_is_test is False"
        if config.current_mode_is_test:
            return get_default_setup_answers(self.config_type)
        user_typed: List = []
        for item in self.items:
            ans = self.get_from_user(item.msg, item.long_msg, item.check_func)
            """ We will transform users answer same / split for now """
            ans = item.transform_func(ans)

            user_typed.append(ans)

        # user_typed = list(map(lambda x: x , user_typed))
        self.user_typed = user_typed
        return tuple(user_typed)


# ----------------------------------------------------GetInputFromUserConfig----------------------
@dataclass
class GetInputFromUserConfig(GetInputFromUserBase):
    """  GetInputFromUserConfig  """
    items: List[InputItem]
    config_type: configType = configType.Config


# ----------------------------------------------------GetInputFromUserSeries----------------------
@dataclass
class GetInputFromUserSeries(GetInputFromUserBase):
    """  GetInputFromUserSeries  """
    items: List[InputItem]
    config_type: configType = configType.Series


# ----------------------------------------------------create_Input_Items----------------------
def create_Input_Items(
        config_type: configType,
        items_from_user: List, explanations: List,
        check_funcs: List[Callable],
        transform_answers: List[Callable],

):
    items: List[InputItem] = []
    for msg, explanation, check_func, transform_func in zip(items_from_user, explanations, check_funcs,
                                                            transform_answers):
        items.append(
                InputItem(
                        msg=msg,
                        long_msg=explanation,
                        check_func=check_func,
                        transform_func=transform_func
                )
        )

    GetInputFromUserClassType = {
            configType.Series: GetInputFromUserSeries,
            configType.Config: GetInputFromUserConfig
    }[config_type]

    GIF = GetInputFromUserClassType(
            items=items, config_type=config_type
    )
    return GIF.process()


from evdspy.EVDSlocal.components.evds_seri_files import EvdsSeri

from typing import List

from evdspy.EVDSlocal.common.folder_name_checks import check_remove_back_slash


def content_from_SetupInputsSeries(SI: SetupInputsSeries):
    """ S E R I E S """
    """
    namedtuple('SetupInputsSeries', 'data_folder subject prefix ecodes frequency formulas aggregateType')

    """

    def create_content(folder, subject, prefix, series_list: Tuple):
        folder_path = check_remove_back_slash(folder)
        # folder_path_series = str(Path().absolute() / folder / 'series.txt')
        abs_path = str(Path().absolute() / folder_path)
        if isinstance(series_list, str) and not series_list.strip():
            series_list = default_answers_series

        if isinstance(series_list, str) and not len(series_list.strip()):
            split_items: Tuple = lambda text: tuple(
                    text.translate(text.maketrans({x: "-" for x in "[,-/\n;]~"})).split("-"))

            series_list: Tuple = split_items(series_list)

        series_list_str: str = "\n".join(series_list)
        r = f"""{mainSepBegin}
foldername : {folder_path}
abs_path : {abs_path} # will check again before saving requests from the server it may be replaced by ...WD.../DataSeries/{folder_path}
subject  : {subject}
prefix   : {prefix}
frequency : {SI.frequency} # {frequency_dict[int(SI.frequency)]}
formulas : {SI.formulas} # {formulas_dict[int(SI.formulas)]}
aggregateType : {SI.aggregateType}
------------SERIES CODES------------------
{series_list_str}
------------/SERIES CODES------------------
{mainSepEnd}
{GSEP}"""
        return r

    return create_content(SI.data_folder, SI.subject, SI.prefix, tuple(SI.ecodes))


def content_from_SetupInputsOptions(SI: SetupInputsConfig):
    """ O P T I O N S  """
    """
    SetupInputsConfig = namedtuple('SetupInputsConfig', 'cache_freq gl_date_start gl_date_end')
    """

    def get_default(item):
        # SingletonOptions().get_valid_value("default_cache")
        # SingletonOptions().get_valid_value("default_start_date")
        # SingletonOptions().get_valid_value("default_end_date")

        cache_freq = SingletonOptions().options_.default_cache
        gl_date_start = SingletonOptions().options_.default_start_date
        gl_date_end = SingletonOptions().options_.default_end_date

        # result = SI.__getattribute__(item)
        # result = SI.__dict__.get(item, None)
        result = getattr(SI, item, None)
        if not result or not result.strip():
            result = locals()[item]
        return result

    def create_content():
        cache_freq = get_default("cache_freq")
        gl_date_start = get_default("gl_date_start")
        gl_date_end = get_default("gl_date_end")

        """ Options """
        r = f"""#Global Options File
# G L O B A L   O P T I O N S   F I L E   -------------------------------------------------------    
cache_freq : {cache_freq}  
gl_date_start : {gl_date_start}   
gl_date_end : {gl_date_end}

{GSEP}"""
        print(r)
        return r

    return create_content()


def content_from_SetupInputsSeries_MainFile(SI: SetupInputsSeries):
    cont = content_from_SetupInputsSeries(SI)
    content = f"{cont}"
    return content


def content_from_SetupInputsOptions_MainFile(SI: SetupInputsConfig):
    cont = content_from_SetupInputsOptions(SI)
    content = f"{cont}"
    return content


# ----------------------------------------------------create_config_series----------------------
def create_config_series(SI: SetupInputsSeries):
    """ Main Aggregated Configuration file for all series """
    config_series_main_file = 'config_series.cfg'
    SetupInputsSeriesRepr = content_from_SetupInputsSeries_MainFile(SI)
    if not is_file(config_series_main_file):
        SetupInputsSeriesRepr = series_title + "\n" + SetupInputsSeriesRepr
    WriteAdd(config_series_main_file, SetupInputsSeriesRepr)

    """ Individual Series ( Each call create only one set of series) """
    create_folder(SI.data_folder)
    series_file_content = content_from_SetupInputsSeries(SI)
    Write(str(Path() / SI.data_folder / 'series.txt'), series_file_content)
    return True


# ----------------------------------------------------create_config_config----------------------
def create_config_config(SI: SetupInputsConfig):
    """ Main Aggregated Configuration file for all USER OPTIONS """
    config_options_main_file = 'options.cfg'
    SetupInputsSeriesRepr = content_from_SetupInputsOptions_MainFile(SI)
    Write(config_options_main_file, SetupInputsSeriesRepr)
    print_with_creating_style("writing")
    print_with_creating_style(SetupInputsSeriesRepr)
    return True


# ----------------------------------------------------             ----------------------

def get_item_for_config_type(obj, config_type):
    return obj[config_type]


# ---------------------------------------------------- InputType ----------------------

@dataclass
class InputType:
    items_from_user: list
    explanations: list
    check_funcs: list
    transform_answers: Union[List[callable], Tuple[callable]]
    applyFunc: Callable  # not from config
    SetupInputs: any  # ...


# ----------------------------------------------------start_setup----------------------


def start_setup(
        config_type: configType = configType.Series,
        onsetup: bool = False,
):
    # assert config_type == configType.Series, "config_type not configType.Series "
    """ S E R I E S """
    Series_obj = InputType(
            items_from_user_series,
            explanations_series,
            check_funcs_series,
            transform_answers_series,
            create_config_series,
            SetupInputsSeries
    )

    """ O P T I O N S """
    Config_obj = InputType(
            items_from_user_config,
            explanations_config,
            check_funcs_options,
            transform_answers_options,
            create_config_config,
            SetupInputsConfig
    )

    instance = {configType.Config: Config_obj, configType.Series: Series_obj}[config_type]
    if not onsetup:
        CIS = create_Input_Items(config_type, instance.items_from_user, instance.explanations, instance.check_funcs,
                                 instance.transform_answers)
        print_with_creating_style(CIS)
    else:

        CIS = tuple("" for _ in items_from_user_config)

    instance.applyFunc(instance.SetupInputs(*CIS))


# ----------------------------------------------------start_setup_series----------------------

def start_setup_series():
    msg = f"""
    This process is going to add new series to yuor configuration file (config_series.cfg)....
    You may also update all info by opening file with your favorite file editor. 
    --------------------------------------
    Program will ask couple of questions to complete your series block on your file.
     
"""

    print_with_creating_style(msg)
    time.sleep(1)
    start_setup(configType.Series)
    return True


# ----------------------------------------------------start_setup_config----------------------


def start_setup_config(onsetup=False):
    file_name_options = "options.cfg"
    msg = f"""
        This process is going to create user options file  (options.cfg)....
        You may also update all info by opening file with your favorite file editor. 
        --------------------------------------
        Program will ask couple of questions to complete your options on your file.

    """
    if onsetup:
        msg = f"""
        creating `{file_name_options}` 
"""

        if file_exists_show_and_return(file_name_options):
            return
    print_with_creating_style(msg)
    time.sleep(1)
    start_setup(configType.Config, onsetup)
    return True


# -----------------------------------------------------------------------------------------
"""
            default_setup
"""


# -----------------------------------------------------------------------------------------


def get_default_setup_answers(config_type: configType = configType.Config):
    default_answers = {
            configType.Series: default_answers_series,
            configType.Config: default_answers_config
    }[config_type]
    return default_answers


def default_setup(config_type: configType = configType.Config):
    print_with_updating_style("Default setup starting....")

    CIS = get_default_setup_answers(config_type)
    create_config_series(SetupInputsSeries(*CIS))
    return True


__all__ = [
        'default_setup',
        'create_config_series',
        'start_setup_series',
        'default_setup',
        'folder_creatable_by_adding',
        'folder_format_check',
        'folder_creatable',
        'get_default_setup_answers'

]



from ..common.common_imports import *
from dataclasses import dataclass
from ..log_classes.log_template import get_debugger

Name_or_False = TypeVar('Name_or_False', str, bool)

from typing import Optional


@dataclass
class Args:
    args: field(default_factory=list)
    # proxy: Name_or_False = False
    proxy: Optional[str] = False
    file_name: Name_or_False = Name_or_False

    def __post_init__(self):
        self.populate_args()

    def populate_args(self):
        self.proxy = self.extract_prop("--proxy")
        self.file_name = self.extract_prop("--file")

    def extract_prop(self, prop):
        for index, item in enumerate(self.args):
            if prop in item:
                if "=" in item:
                    v = item.split("=")[1]
                else:
                    v = self.args[index + 1]
                return v
        return False


import sys

test_args = Args(sys.argv)


deb("test_args" + ",".join(list(sys.argv)))

# print(debug.log_file_name, debug.log_file_path)
# print(debug)




from evdspy.EVDSlocal.requests_.ev_request import *

from ..tests.core_options_test import *


from evdspy.EVDSlocal.manual_requests.prepare import PrepareUrl

def test_prep(capsys):
    with capsys.disabled():
        p = PrepareUrl(series=('TP_ODEMGZS_NORVEC-8',), api_key='api_key')
        print(p.get_data())



from evdspy.EVDSlocal.common.common_imports import *

from evdspy.EVDSlocal.utils.utils_general import *
from evdspy.EVDSlocal.initial.start_options import *
from ..common.colors import *

from evdspy.EVDSlocal.initial.start_options import default_data_folder_name

# dirs = ["pickles", "output", options_folder_name]
# dirs = ["pickles", options_folder_name]
dirs = ["pickles"]
dirs = dirs + [default_data_folder_name]

cur_folder = Path()

import time


def check_folders_setup_necessary():
    for item in dirs:
        folder_ = cur_folder.absolute() / item
        if not folder_.is_dir():
            return True
    return False

def check_setup():
    return not check_folders_setup_necessary()


def setup_folders():
    if not check_folders_setup_necessary():
        deb(f"initial folders checked...{dirs} ")
        print_with_success_style("Setup => checked...")
        time.sleep(1)
        return

    for item in dirs:
        # folder_ = cur_folder.absolute() / ".." / item
        folder_ = cur_folder.absolute() / item

        print_with_creating_style(f"Creating directory...{folder_}")
        create_directory(str(folder_))
        time.sleep(0.8)



# ------------------------------------------------------
#
#       M a i n    F i l e
#                           package: evdspy @2022
# ------------------------------------------------------







from evdspy.EVDSlocal.main_ import *
from evdspy.EVDSlocal.initial.try_loading_initial_commands import *
from evdspy.EVDSlocal.initial.load_commands_ import *



# from functools import lru_cache
import shutil

from ..common.common_imports import *
from evdspy.EVDSlocal.utils.utils_general import *
from ..components.api_params import get_enum_with_value
from ..components.options_class import load_options, SingletonOptions
from ..common.url_clean import remove_api_key
from evdspy.EVDSlocal.series_format.stats.save_stats_of_requests import Report
from evdspy.EVDSlocal.config.apikey_class import ApikeyClass
from typing import Union

# -----------------------------------------------------------------------------
test_result_file_name = str(get_current_dir() / ".." / "requests_" / "test_reg_result")
arg_sep = "_argSEP_"
p = Path()
pickle_folder = p.absolute() / 'pickles'


# -----------------------------------------------------------------------------


class CacheDecider(Enum):
    day = date.today()
    hour = datetime.now().strftime("%H")
    nocache = datetime.now().strftime("%H:%M:%S")
    default = date.today()


options_ = load_options()


def get_default_cache():
    obj = {
            "daily": CacheDecider.day,
            "hourly": CacheDecider.hour,
            'nocache': CacheDecider.nocache,
            'default': CacheDecider.day

    }
    return obj.get(options_.default_cache, CacheDecider.day)


def cache_represent(choice: CacheDecider):
    obj = {
            CacheDecider.day: "daily",
            CacheDecider.hour: "hourly",
            CacheDecider.nocache: 'nocache',
            CacheDecider.default: 'daily'
    }
    return obj[choice]


@dataclass
class MyCache:
    decider: Union[CacheDecider, None] = None  # post init

    # get_default_cache()  # get_default_cache()  # CacheDecider.hour
    report_ = Report("stats_requests.txt")

    def __post_init__(self):
        default_cache = SingletonOptions().get_valid_value("default_cache")
        self.decider = get_enum_with_value(key=default_cache, enum_=CacheDecider, default_value=CacheDecider.day)

    def update_rules_of_cache(self):
        if ApikeyClass().now_testing_is_key_is_valid:
            msg = "since this looks like a test, Program will be changing cache rulse to nocahce at all..."
            print(msg)
            deb(msg)

    def report(self, content=None):
        if content is None:
            content = f"""

            Cache Info : Current request cache method caches {cache_represent(self.decider)}

"""
        self.report_.add(content)

    def prepare_report_for_cache(self, func, args):
        self.report("Cache loaded...")
        str_args = (x for x in args if isinstance(x, str))
        str_args = tuple(map(remove_api_key, str_args))
        content = f"<function:{func.__name__}>" ", \n".join(str_args)
        self.report(content)

    def cache(self, func):
        self.update_rules_of_cache()

        @functools.wraps(func)
        def wrapper_decorator(*args, **kwargs):
            if self.check_cache(func, *args):
                value = self.load_cache(func, *args)
                self.prepare_report_for_cache(func, args)

            else:
                value = func(*args, **kwargs)
                self.save_cache(func, *args, data=value)
            return value

        return wrapper_decorator

    def sort_safe(self, items):
        items = list(items)
        items = sorted(items)
        return items

    def safe_serialize(self, *items):
        safe_list = []
        for item in items:
            if isinstance(item, str):
                safe_list.append(item)

        safe_list = self.sort_safe(safe_list) + [str(self.decider)]
        parameters = arg_sep.join(safe_list)
        return parameters

    def get_hash_alt1(self, st):
        import hashlib, base64
        d = hashlib.md5(bytes(st, encoding="utf-8")).digest()
        d = base64.urlsafe_b64encode(d).decode('ascii')
        return d

    def get_func_hash(self, f, *args):
        parameters = self.safe_serialize(*args)
        f_hash = f"{f.__name__}__{parameters}"
        f_hash = self.get_hash_alt1(f_hash)
        return f_hash

    def check_cache(self, f, *args):
        """added below to provide an exception when we check for valid key """
        self.update_rules_of_cache()

        f_hash = self.get_func_hash(f, *args)
        f_hash_path = rf"{pickle_folder}\\{f_hash}"
        return check_pickle(f_hash_path)

    def load_cache(self, f, *args):
        f_hash = self.get_func_hash(f, *args)
        f_hash_path = rf"{pickle_folder}\\{f_hash}"
        return load_pickle(f_hash_path)

    def save_cache(self, f, *args, data=None):
        f_hash = self.get_func_hash(f, *args)
        f_hash_path = f"{pickle_folder}\\{f_hash}"
        save_pickle(f_hash_path, data)


def check_pickle(file_name: str) -> bool:
    return os.path.isfile(file_name + ".pickle")


def load_pickle(file_name: str) -> Union[bool, requests.models.Response]:
    file_name = file_name + ".pickle"
    file_name_path = Path(file_name)

    if not file_name_path.is_file():
        print(file_name_path)
        return False
    with open(file_name_path, "rb") as infile:
        test_dict_reconstructed: requests.models.Response = pickle.load(infile)

        # exit()

    # f = bound(print, f"{file_name_path} pickle loaded...")
    # functools.partial(print, f"{file_name} pickle loaded...")
    # delay(1, f)
    msg = f"""
    
    ---------------------------------
    
    
    Recent cache was found for this request phrase.
    ---------------------------------
        It will be loaded instead of making a new redundant request 
        You may change cache options in options file.
    ---------------------------------
        cache loaded...({file_name_path})
"""

    print(msg)

    return test_dict_reconstructed


def delete_cache_folder():
    path = Path(pickle_folder)
    if not path.is_dir():
        return True
    try:
        # Path.rmdir(path)
        # os.remove(path)
        shutil.rmtree(path)
        return True
    except Exception as exc:
        print(exc)
    return False


def save_pickle(file_name, data):
    # print("saving", file_name)
    msg = f"""
        
    ---------------------------------
    
    
        Request was successfull...
        ---------------------------------
            Cache will be saved for this result. Next time program will be
            checking if this cache is new enough to load for the new request 
            You may change cache options in options file 
            ---------------------------------
            cache saved ...({file_name})
            
    """

    file_name_path = file_name + ".pickle"
    try:
        save_pickle_helper(file_name_path, data)
    except:

        print("Cache folder may be deleted.\n setup_now() inorder to create...")
        # if callable("setup_now"):
        #     setup_now()
        #     save_pickle_helper(file_name_path, data)


def save_pickle_helper(file_name_path, data):
    with open(file_name_path, "wb") as outfile:
        pickle.dump(data, outfile)
    f = bound(print, f"{file_name_path} pickle saved...")
    delay(1, f)


def save_pickle_for_test(data):
    save_pickle(test_result_file_name, data)


def load_test_pickle():
    return load_pickle(test_result_file_name)


def lru_cache_patched(maxsize=128, typed=False):
    import weakref
    def decorator(func):
        @functools.wraps(func)
        def wrapped_func(self, *args, **kwargs):
            self_weak = weakref.ref(self)

            @functools.wraps(func)
            @functools.lru_cache(maxsize=maxsize, typed=typed)
            def cached_method(*args, **kwargs):
                return func(self_weak(), *args, **kwargs)

            setattr(self, func.__name__, cached_method)
            return cached_method(*args, **kwargs)

        return wrapped_func

    if callable(maxsize) and isinstance(typed, bool):
        func, maxsize = maxsize, 128
        return decorator(func)
    return decorator


__all__ = [
        'lru_cache_patched',
        'MyCache',
        'save_pickle_for_test',
        'load_test_pickle'
]



from evdspy.EVDSlocal.messages.error_classes import ApiKeyNotSetError


class BaseException_Request(Exception):
    """BaseException_Request"""


class Req_NotFound(BaseException_Request):
    """Req_NotFound"""


class Req_Unauthorized(BaseException_Request):
    """Req_NotFound"""


class Req_Forbidden(BaseException_Request):
    """Req_NotFound"""


class Req_BadRequest(BaseException_Request):
    """Req_NotFound"""


class Req_TooMany(BaseException_Request):
    """Req_TooMany"""


class Internal(BaseException_Request):
    """Internal"""
    # this one needs extra checks
    # when api key set but not a valid key, it becomes extra cause for this one
    # TODO
    # simple URL check when any new api key was set which only breaks due to api key
    # TODO
    # In order to avoid correct api key and bad URL requests
    # create another cache folder to check if mistaken URL lists includes current URL before requesting
    # to avoid too many abd requests from the server
    #       => Another TODO to diminish the last one
    #               checks for params coherence


class Bad_Gateway(BaseException_Request):
    """Bad_Gateway"""


class ServiceUnavailable(BaseException_Request):
    """ServiceUnavailable"""


class GatewayNotImplemented(BaseException_Request):
    """GatewayNotImplemented"""


REQUEST_ERROR_CODES = {
        400: Req_BadRequest,
        401: Req_Unauthorized,
        403: Req_Forbidden,
        404: Req_NotFound,
        429: Req_TooMany,
        500: Internal,
        502: Bad_Gateway,
        503: ServiceUnavailable,
        504: GatewayNotImplemented
}



""" S T A R T   O P T I O N S """
#
#
#   see SingletonOptions.py
#
from ..common.common_imports import *
from pathlib import Path


def check_if_this_is_pytest():
    def check():
        path_ = Path(sys.argv[0])
        return "pytest" in str(path_.stem)

    if len(sys.argv) > 0 and check():
        return True
    else:

        return False

#
#
#
if True:
    default_series_file_name = "config_series.cfg"
    default_data_folder_name = "SeriesData"
    Avoid_Absolute_Paths_ = True
    Default_Prefix_ = "EVPY_"
    # options_folder_name = r"IO"
    DEBUG_LOG_CANCEL = False
    DEGUB_NOTICE = True
    DEBUG_PRINT = False
    default_arch_folder_name = "APIKEY_FOLDER"
    default_arch_file_name = "arch.txt"
    default_cache = "daily"  # nocache / hourly / monthly
    default_end_date = "01-12-2030"
    default_start_date = "01-01-2019"
    import os

    # USERNAME = os.getenv("USERNAME")  # or hard coded "Username"
#
#
#
#
#
#
#
#
#
#
""" / S T A R T   O P T I O N S """

# pytest will set this True
current_mode_is_test = check_if_this_is_pytest()

""" GLOBALS """



from dataclasses import dataclass, field
from enum import Enum
from evdspy.EVDSlocal.components.url_class import URLClass
class RequestType(Enum):
    Request = "request"
    Cache = "cache"
@dataclass
class RequestOrCacheResultInfo():
    safe_url: str = ""
    request_type: RequestType = RequestType.Request
    columns: field(default_factory=tuple) = tuple()
    excel_saved: bool = False
    url_instance: URLClass = URLClass([])
    def get_data(self):
        content = f"""
url : {self.safe_url}
request or cache : {self.request_type.value}
columns : {self.columns}
excel file was saved ? : {self.excel_saved}
"""
        return content



from .cmd_formats import *
from ..common.common_imports import *
from ..common.files import Read, ReadBytes


def display_help_messages_md():
    def ReadLocal(filename : str ):
        long_des = ""

        with open(filename, "r") as file_:
            long_des = file_.read()
        return long_des
    # return
    from rich.console import Console
    from rich.markdown import Markdown
    print(Path().parent)
    content_md = ReadLocal(str(Path() / Path().parent / "README.md"))
    console = Console()
    # print(content_md)
    md = Markdown(content_md)
    console.print(md)
    return content_md
    # content_md = str(content_md).encode('utf8')
    # md = Markdown(str(content_md))
    # console.print(md)


def display_help_messages():
    """ Some help to show some useful commands of project """
    ##############  commands #################
    msg = rf"""


--------importing -----
from evdspy  import *

-------------

------------ Below you may find some useful commands to check your installation of evdspy
             You may modify `IO\options.py` file with your preferences
                            `series.txt` file with your data series codes

menu() :
-------------
            Quickly setup your work environment run commands from the menu. \
            Currently you may choose one below or create your own functions and \
            add to this list following instructions on Readme.md
            **************************************************
                                M E N U
            **************************************************
                           1. check setup
                           2. setup
                           3. create user options file
                           4. create series file
                           5. add new series group
                           6. get data
                           7. help
                           8. show api key
                           9. save api key to file
                           10. exit (from the menu)


10.exit (#Console#)
            Choosing 10 (Exit from the menu) you may continue working with commands summarized below
            or importing additional commands from the documentation. 
help_evds() :
-------------
    to see a list of initial commands  
    to check setup and reinitialize some installations.

check():
-------------
    see current installation status

easy_setup()   :
-------------
    # creates necessary folders such as `pickles`, `IO` for the environment

    easy_setup()
    
setup_steps()   :
-------------
    # this function helps you setup your project by asking your preferences step by step
    setup_steps()
    setup()

get():
-------------
    # this will check for your current series.txt file 
    # if proper data series codes are given it will either download them
    # or use latest cache from your local environment 
    # to provide other cache options such as nocache / daily / hourly you may change your 
    # defaults or give arguments such as 
        
        get(cache="daily")
        get(cache = "nocache")
        or to go with default from your option file 
        get()

save( key = "xxxxyyy"):
-------------
    # to get data quickly for this session you may set  apikey which will
    # be deleted before the session ends,  
    # if you would like to provide a file adress which has your api key 
    # see the other option setting your key globally 
    # => change your options.py file in IO folder 
    set_apikey("MyApiKey")
save():
-------------
    When you call it with not argument program will ask for your key and do the same 
    above 
create_series_file():
-------------
    # creates example `series.txt` evdspy input file (EIF) formatted 
    # you may modify it accordingly, you may user separator of -- between different groups  

    create_series_file()
    # or
    csf()

menu() :
-------------
            Quickly setup your work environment run commands from the menu. 
            Currently you may choose one below or create your own functions and 
            add to this list following instructions on Readme.md
            **************************************************
                                M E N U
            **************************************************
                           1. check setup
                           2. setup
                           3. create user options file
                           4. create series file
                           5. add new series group
                           6. get data
                           7. help
                           8. show api key
                           9. save api key to file
                           10. exit (from the menu)\

-----------------------------------------------------------------------
# from evdspy import *         #(if you have not imported evdspy yet )
menu()                         # to see the menu() 
-----------------------------------------------------------------------
"""
    ############## END of commands #################

    ################ p r i n t ###############
    print(msg)
    ################            #######



from ..common.prog import rich_sim
from ..messages.error_messages import *
from ..messages.error_classes import *
from evdspy.EVDSlocal.initial.load_modules import LoadModulesClass
from ..examples.examples_onload import *
from ..config.config import config
from ..initial_setup.initial_setups import *
from ..console.menu import *
from ..requests_.my_cache import delete_cache_folder
from ..setup_project.user_setup import start_setup_series, start_setup_config
from evdspy.EVDSlocal.utils.utils_general import *

##### LoadModulesClass ##############
deb("load_commands was loaded")
from evdspy.EVDSlocal.initial_setup.setup_folders import check_folders_setup_necessary, check_setup
from ..common.colors import *


def check_setup_runtime():
    return not check_folders_setup_necessary()


def easy_setup():
    setup()


def setup_again():
    setup()


def setup():
    SetupInitial().setup()
    start_setup_config(onsetup=True)
    create_series_text_example(onsetup=True)


def help_evds():
    display_help_messages()


def help_():
    return help_evds()


def h():
    return help_evds()


def help_evdspy():
    return help_evds()


def check():
    print_with_updating_style("checking....")
    lmc = LoadModulesClass()
    lmc.check()


def show():
    lmc = LoadModulesClass()
    lmc.series_from_file()
    lmc.summary()


def get_input(msg):
    if config.current_mode_is_test:
        return "test"
    print_get_input_style(msg)
    ans = input()
    return str(ans)


def wait(num: int):
    time.sleep(num)


from evdspy.EVDSlocal.initial_setup.api_key_save import save_api_key_to_file
from typing import Callable


def save_api_key_to_file_main(api_key):
    @dataclass
    class transfer():
        get_input: Callable = get_input
        wait: Callable = wait

    save_api_key_to_file(instance=transfer(), api_key_answer=api_key)


def check_if_api_set_before_get():
    if not check_api_key():
        print_with_failure_style(api_key_not_set_msg)
        wait(2)
        if not get_input("yes/no ?  ").lower() in ["yes", "y"]:
            wait(2)
            return
        # below the function checks
        # if api key is valid then saves it
        if not set_apikey_input():
            return False
        print_with_success_style("finished")
        return True
    return True


def get():
    if not check_setup():
        print_with_updating_style("Checking setup...")
        # setup_folders()
        SetupInitial().setup()
    if not check_if_api_set_before_get():
        return
    try:
        lmc = LoadModulesClass()
        lmc.series_from_file()
        lmc.summary()
        # try:
        #     lmc.evds_list[0].df.plot(x='TP.ODEMGZS.BDTTOPLAM', y='Index', style='o')
        # except:
        #     pass
    except SeriesFileDoesNotExists:
        print("...")
        # print(SeriesFileDoesNotExists().message)


def get_df_test():
    LoadModulesClass().series_from_file()
    LoadModulesClass().summary()
    return LoadModulesClass().evds_list[0].df


from ..config.apikey_class import *
from evdspy.EVDSlocal.requests_.lower_energy import *


def check_api_key_with_low_energy(api_key_from_user):
    # check api key with cache
    # send => basic_for_test > performance.lower_energy
    if not apikey_works(api_key_from_user):
        print_with_failure_style(f"Api key did not work! => {api_key_from_user}")
        return False
    print_with_success_style(f"Api key works => {api_key_from_user}")
    return True


def obvious_checks_of_api_key(api_key: str) -> bool:
    conds: tuple = (isinstance(api_key, str), not len(api_key) < 10,)
    return all(conds)


def save_apikey(api_key: str = None):
    if api_key is None or not isinstance(api_key, str):
        return set_apikey_input()
    set_apikey_input(api_key)


def set_apikey_input(api_key=None, from_console=False):
    exit_words = ["e", "exit"]
    if config.current_mode_is_test:
        print_with_updating_style("pytest returns ")
        ans = "test"
    else:
        if not from_console:
            ans = get_input("Api key [ `e` or `exit` to cancel] : ")
        else:
            ans = api_key
    api_key_from_user = str(ans)
    if api_key_from_user.lower() in exit_words:
        print_with_updating_style("exited")
        return False
    if not obvious_checks_of_api_key(api_key_from_user):
        print_with_failure_style(
                f"The text you entered does not meet criteria to be an valid api key. \n"
                f"{indent}(length of api key cannot be less then 10) {api_key_from_user}")
        return False
    """this function will save redundant requests by catching earlier check results """
    if not check_api_key_with_low_energy(api_key_from_user):
        return False
    """"""
    """ SAVE IT TO FILE """
    save_api_key_to_file_main(api_key_from_user)
    """ SAVE IT FOR RUNTIME """
    ApikeyClass().set_api_key_runtime(value=api_key_from_user)
    show_apikey()
    return True


from evdspy.EVDSlocal.initial_setup.api_key_save import check_api_key_on_load


def check_api_key():
    check_api_key_on_load()
    apikeyobj = ApikeyClass()
    return apikeyobj.get_valid_api_key()


def show_apikey():
    apikeyobj = ApikeyClass()
    # k = apikeyobj.get_api_keys_dict()
    valid = apikeyobj.get_valid_api_key()
    disp = f"""
    valid api key : {valid}
    
"""
    print_with_success_style(disp)


from ..series_format.series_creator import *


def create_series_file():
    return create_series_text_example()


def examples():
    examples_get(get)


def csf():
    return create_series_file()


def setup_series():
    start_setup_series()


def setup_user_options():
    start_setup_config(onsetup=False)


def setup_series_steps():
    return setup_series()


def menu(*args, **kw):
    menu_helper()
    # if CurrentState().menu_will_open:
    #     menu_helper()


def menu_onload():
    if CurrentState().menu_will_open:
        menu_helper()


def show_menu():
    menu_helper()


def menu_helper():
    # TODO config
    global menu_already_displayed
    menu_already_displayed = True
    menu_display()
    # if CurrentState().menu_will_open:
    #     menu_display()


def console():
    return -1


def get_develop_vers_main():
    # print_with_updating_style(str(Path.cwd()))
    parent = Path(__file__).parent
    v = Read(Path(parent / ".." / "version__.py"))  # EVDSlocal/ version.py
    return v


def version():
    v = get_develop_vers_main()
    import site
    s = site.getsitepackages()
    site_pack = ""
    f = ""
    if len(s) > 1:
        site_pack = s[1]
        f = "from"
    print_with_success_style(f"evdspy {v} {f} ")
    print_with_info_style(f"evdspy {v} {f} {site_pack}")
    v = f"evdspy {v} {f} {site_pack}"
    return v


def remove_cache():
    # TODO activate
    msg = "This will attempt to delete cache files in order to make new requests. \n " \
          "If you created your options file you may choose caching period to a more frequent one. \n" \
          "this function was added to force making new requests ignoring recent ones. \n" \
          "Proceed? (Y/n)"
    ans = get_input(msg)
    if not ans in ["y", "Y"]:
        print_with_failure_style("exited without removing the cache")
        return
    print_with_success_style("removing cache...")

    if delete_cache_folder():
        rich_sim(3, "processing...")
        print_with_success_style("cahce folder removed...")
    else:
        print_with_failure_style("cahce folder coould not be removed...")


def menu_display():
    funcs = [
            ("check setup", check),
            ("setup", easy_setup),
            ("create user options file", setup_user_options),
            ("create series file", create_series_file),
            ("add new series group ", setup_series_steps),
            ("get data", get),
            # ("show summary", show),
            ("help", help_),
            ("show api key", show_apikey),
            ("save api key to file", set_apikey_input),
            ("remove cache folders", remove_cache),
            ("version", version),
    ]
    menu_items = list(map(lambda x: MenuItem(x[1], x[0]), funcs))
    MenuMaker(
            menu_items=menu_items,
            exit_menu_call_back=help_
    ).display()


# ---------------------------------------------------------------MAIN LOAD
""" MAIN LOAD """
from evdspy.EVDSlocal.state.current_state import CurrentState

current_state = CurrentState()


def save(*args):
    return save_apikey(*args)


# ---------------------------------------------------------------/ MAIN LOAD

__all__ = [
        'easy_setup',
        'setup',
        'setup_series',
        "setup_series_steps",
        'help_evds',
        'check',
        'get',
        'h',
        'help_evdspy',
        'help_',
        'examples',
        'create_series_file',
        'csf',
        'show',
        'get_df_test',
        'console',
        'menu',
        'menu_onload',
        'version',
        'save_apikey',
        'save',
        'remove_cache'
]



from evdspy.EVDSlocal.tests.test_evds_classes import *
from evdspy.EVDSlocal.tests.test_main_load import *

from evdspy.EVDSlocal.tests.test_project_setup import *
from evdspy.EVDSlocal.tests.test_log_session import *
from evdspy.EVDSlocal.tests.test_requests import *

from evdspy.EVDSlocal.tests.test_apikey_checks import *
#
# from .tests.test_series_files import *
# from .tests.test_manual_series import *



#### 1 ###############
import base64
import functools
import locale
# from babel.numbers import format_decimal
import pickle
import functools
import weakref
import functools
import os.path
from pathlib import Path

indent = " " * 15
ENCODING = "utf-8"
#### 2 ###############
import os

#### GLOBALS ###############

#### -- ###############
from datetime import date

today = date.today()
stoday = str(today)

import time


def do_if_callable(f):
    if callable(f):
        f()


from pathlib import Path

def URL_temizle(url: str):
    import string
    return url.translate({ord(c): None for c in string.whitespace})


def get_current_dir():
    return Path(__file__).parent


def get_current_dir2(pat):
    d = pat.absolute()

    return d


def delay(s: int, f1=None, f2=None):
    do_if_callable(f1)
    time.sleep(s)
    do_if_callable(f2)


def ro(amount: float):
    return round(amount)


def bound(f, *args):
    return functools.partial(f, *args)


def create_directory(adres: str) -> None:
    import os
    if not os.path.isdir(adres):
        os.makedirs(adres)


def get_api_key_from_file(file_name):
    lines = False
    file_name_path = Path(file_name)
    if not file_name_path.is_file():
        return False

    with open(file_name, encoding=ENCODING) as file:
        lines = file.readlines()
    if lines is False:
        return False
    return str(lines[0]).strip()  # .split(":")[1]


def global_var_api_key():
    import os
    return "This version does not read global EVDS"
    API_KEY_NAME = "EVDS_API_KEY"
    return os.getenv(API_KEY_NAME)


def get_proxy_from_file(file_name):
    lines = False
    if not file_name:
        return False
    file_name_path = Path(file_name)
    if not file_name_path.is_file():
        return False
    with open(file_name_path, encoding=ENCODING) as file:
        lines = file.readlines()
    if lines is False:
        return False
    return lines[0]  # ":".join(lines[0].split(":")[1:])


def reducer(liste):
    functools.reduce(lambda a, b: a + b, liste)


def reducer_prop(liste, prop="guncel_tutar"):
    # print(liste ,prop , "---" )
    return functools.reduce(lambda a, b: a + b,
                            [getattr(item, prop) for item in liste])


def f2(amount):
    return '{:7,.2f}'.format(amount)


def f0(amount):
    return '{:7,.0f}'.format(amount)


def get_random_hash(num=5):
    import random
    import string
    letters = string.ascii_letters
    randomF = ''.join(random.choice(letters) for i in range(num))
    return randomF



import evdspy
from evdspy.EVDSlocal.session.session_ import *
from evdspy.EVDSlocal.log_classes.log_template import *

session = get_session()


def test_get_session(capsys):
    s = get_session()
    with capsys.disabled():
        print(f"\nSession : {s.hash}")


def test_log():
    deb, deb2, debug = get_debugger()



import evdspy
from evdspy.EVDSlocal.series_format.series_creator import *
from evdspy.EVDSlocal.components.evds_seri_files import *
from evdspy.EVDSlocal.setup_project.user_setup import start_setup_series, create_config_series, SetupInputsSeries, \
    start_setup_config, folder_format_check, folder_creatable, folder_creatable_by_adding, default_setup, \
    get_default_setup_answers, configType

from ..tests.core_options_test import *


# from evdspy.EVDSlocal.setup_project.user_setup import  *


def test_start_setup(capsys):

    CIS = get_default_setup_answers(configType.Series)
    with capsys.disabled():
        if verbose:
            print(SetupInputsSeries(*CIS).__class__.__name__)
            print(type(SetupInputsSeries(*CIS)))
    assert "SetupInputsSeries" == SetupInputsSeries(*CIS).__class__.__name__, "SetupInputs(*CIS).__class__.__name__"
    assert issubclass(SetupInputsSeries, SetupInputsSeries), "SetupInputs is not  subclass of SetupInputs"


def test_create_config_series(capsys):

    return # this one is ok. Avoided adding new content to real file
    CIS = get_default_setup_answers(configType.Series)
    ret = create_config_series(SetupInputsSeries(*CIS))
    assert ret is True, "test_create_config_series failed"


def test_folder_creatable_by_adding():
    assert folder_creatable_by_adding("=&)%=") is False
    assert folder_creatable_by_adding(r"C:\Users\UserX\PycharmProjects\evdspy\evdspy44\output44") is False


def test_folder_format_check():
    assert folder_format_check("=&)%=") is False
    assert folder_format_check("xxyy") is True
    assert folder_format_check(r"C:\Users\UserX\PycharmProjects\evdspy\evdspy44\output") is False


# def test_start_setup_series():
#     start_setup_series() is True

def test_get_default_setup_answers():
    isinstance(get_default_setup_answers(), list), "get_default_setup_answers"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['evdspy',
 'evdspy.EVDSlocal',
 'evdspy.EVDSlocal.common',
 'evdspy.EVDSlocal.components',
 'evdspy.EVDSlocal.config',
 'evdspy.EVDSlocal.console',
 'evdspy.EVDSlocal.examples',
 'evdspy.EVDSlocal.helper',
 'evdspy.EVDSlocal.initial',
 'evdspy.EVDSlocal.initial_setup',
 'evdspy.EVDSlocal.log_classes',
 'evdspy.EVDSlocal.manual_requests',
 'evdspy.EVDSlocal.messages',
 'evdspy.EVDSlocal.requests_',
 'evdspy.EVDSlocal.series_format',
 'evdspy.EVDSlocal.series_format.stats',
 'evdspy.EVDSlocal.session',
 'evdspy.EVDSlocal.setup_project',
 'evdspy.EVDSlocal.state',
 'evdspy.EVDSlocal.stats',
 'evdspy.EVDSlocal.tests',
 'evdspy.EVDSlocal.utils']

package_data = \
{'': ['*']}

install_requires = \
['openpyxl>=3.0.10,<4.0.0',
 'pandas>=1.5.0,<2.0.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.5.1,<13.0.0']

setup_kwargs = {
    'name': 'evdspy',
    'version': '1.0.17.2',
    'description': 'Get data from EVDS API and organize your local work environment with systematical approach. Using caching facility, this package makes less request from the server.',
    'long_description': '\n[![Python package](https://github.com/SermetPekin/evdspy/actions/workflows/python-package.yml/badge.svg)](https://github.com/SermetPekin/evdspy/actions/workflows/python-package.yml)\n[![Python 3.8](https://img.shields.io/badge/python-3.8-blue.svg)](https://www.python.org/downloads/release/python-380/)\n[![Python 3.9](https://img.shields.io/badge/python-3.9-blue.svg)](https://www.python.org/downloads/release/python-390/)\n[![Python 3.10](https://img.shields.io/badge/python-3.10-blue.svg)](https://www.python.org/downloads/release/python-310/)\n\n\n## evdspy\n\n### installation\n\n    pip install evdspy\n\n### importing\n\n    from evdspy import *  \n    check()\n    get()\n    help_evds()\n\nor\n\n    import evdspy as ev \n    ev.check()\n    ev.get()\n    ev.help_evds()\n\n### menu\n\n    from evdspy import *\n    menu()\n\nor\n\n    import evdspy as ev\n    ev.menu()\n\n    Menu function will display a friendly menu to setup project, creating output folders and some setup files to\n    create some set of series to make a request and download from EVDS server save your api key to get data from EVDS.\n    Than it will convert this data to a pandas dataframe and create some folders on your local area.\n\n\n### menu()\n\n**************************************************\n\n                    M E N U\n\n**************************************************\n\n               1. check setup\n               2. setup\n               3. create user options file\n               4. create series file\n               5. add new series group\n               6. get data\n               7. help\n               8. show api key\n               9. save api key to file\n               10. console\n                                  Selection ?\n\n#### FROM THE CONSOLE\n### cache choice (default:daily)\n\n    # hourly :  no new request within an hour with same URL combination\n    # nocache: new request upon each call without checking cache results\n    # daily : program will use local data requested earlier if data was saved same day.    \n\n    create_series_file()\n\nor\n\n    csf()\n\nor from selection menu choose create series file (config_series.cfg) option.\n\nWith this command program will create file similar to below. You may later add new series info\nor modify this file or delete and create a new on from menu or console using commands summarized in this file.\n\n#### config_series.cfg content example\n\n    #Series_config_file    \n    E V D S P Y  _  C O N F I G  _  F I L E  ---------------------------------------------\n    #\n    # This file will be used by evdspy package (python) in order to help updating \n    # your series. \n    # Script will be adding this file when you setup a new project.\n    # Deleting or modifying its content may require to setup configuration from the beginning\n    # ----------------------------------------------------------------------------------------\n    #\n    #About alternative params \n    # ----------------------------------------------------------------------------------------\n    \n    \n              Frequencies\n              -----------------\n    \n              Daily: 1\n              Business: 2\n              Weekly(Friday): 3\n              Twicemonthly: 4\n              Monthly: 5\n              Quarterly: 6\n              Semiannual: 7\n              Annual: 8\n    \n    \n              `Formulas`s\n              -----------------\n    \n              Level: 0\n              Percentage change: 1\n              Difference: 2\n              Year-to-year Percent Change: 3\n              Year-to-year Differences: 4\n              Percentage Change Compared to End-of-Previous Year: 5\n              Difference Compared to End-of-Previous Year : 6\n              Moving Average: 7\n              Moving Sum: 8\n    \n              Aggregate types\n              -----------------\n    \n              Average: avg,\n              Minimum: min,\n              Maximum: max\n              Beginning: first,\n              End: last,\n              Cumulative: sum\n    \n    \n    #Begin_series\n    \n    ---Series---------------------------------\n    foldername : visitors\\annual\n    abs_path : visitors\\annual \n    subject  : visitors\n    prefix   : EVPY_\n    frequency : 8 # annually\n    formulas : 0 # Level\n    aggregateType : avg\n    ------------SERIES CODES------------------\n    TP.ODEMGZS.BDTTOPLAM\n    TP.ODEMGZS.ABD\n    TP.ODEMGZS.ARJANTIN\n    TP.ODEMGZS.BREZILYA\n    TP.ODEMGZS.KANADA\n    TP.ODEMGZS.KOLOMBIYA\n    TP.ODEMGZS.MEKSIKA\n    TP.ODEMGZS.SILI\n    ------------/SERIES CODES------------------\n    ---/Series---------------------------------\n    --++--\n    \n    ---Series---------------------------------\n    foldername : visitors\\monthly\n    abs_path : C:\\Users\\User\\SeriesData\\visitors\\monthly \n    subject  : visitors\n    prefix   : EVPY_\n    frequency : 5 # Monthly\n    formulas : 0 # Level\n    aggregateType : avg\n    ------------SERIES CODES------------------\n    TP.ODEMGZS.BDTTOPLAM\n    TP.ODEMGZS.ABD\n    TP.ODEMGZS.ARJANTIN\n    TP.ODEMGZS.BREZILYA\n    TP.ODEMGZS.KANADA\n    TP.ODEMGZS.KOLOMBIYA\n    TP.ODEMGZS.MEKSIKA\n    TP.ODEMGZS.SILI\n    ------------/SERIES CODES------------------\n    ---/Series---------------------------------\n    --++--\n\n### initial commands\n\n    from evdspy.main import * \n\n#### help_evds():\n\n    see a list of popular commands of this package to create setup folder and files, and request data.\n    \n\n#### check():\n    check setup and create required folders and see current installation status.\n\n#### setup_now()   :\n\n    creates folders and files\n    ____Folders_______________\n            `pickles` \n                will be used to store some request results to ovoid redundant requests from the EVDS api \n    \n            `SeriesData` \n                to save results of requests or caches to an excel file using information on user option files\n    ____Files_______________\n            `options.cfg`\n                a text file consisting global user options such as start date, end date and caching period.\n\n        \n            `config_series.cfg`\n                this file consists information regarding individual sets of series. From the menu user can add\n            new series that will be requesting from the server. Program will produce on for example and this file\n            can be modified and new sets of series can be added following the example format.\n            \n    from evdspy.main import *\n    setup_now()\n\n\n#### get():\n\n    # this will check for your current series.txt file\n    # if proper data series codes are given it will either download them\n    # or use the latest cache from your local environment\n    # to provide other cache options such as nocache / daily / hourly you may change your\n    # defaults or give arguments such as\n\n        get()\n\n\n#### save( key = "xxxxyyy"):\n\n    Program will store your api key in your environment in a safe folder\n    called APIKEY_FOLDER\n    and only use it when you run a new request which was not requested \n    recently depending on your cache preference.\n\n.\n\n       save_apikey("MyApiKey")\n\n#### save()\n\n        When you call it with not argument program will ask for your key and do the same \n    above \n\n#### create_series_file()  or csf() :\n\n--------------------------------\n\n    # creates example `config_series.cfg` file on your work environment. evdspy input file (EIF) formatted \n    # you may modify it according to your preferences.  \n\n--------------------------------\n\n    create_series_file()\n    # or\n    csf()\n\n# MENU\n### menu()\n\n**************************************************\n\n                    M E N U\n\n**************************************************\n\n               1. check setup\n               2. setup\n               3. create user options file\n               4. create series file\n               5. add new series group\n               6. get data\n               7. help\n               8. show api key\n               9. save api key to file\n               10. exit (from menu to console)\n    ......................... Selection ?',
    'author': 'Sermet Pekin',
    'author_email': 'sermet.pekin@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/SermetPekin/evdspy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from evdspy.EVDSlocal.requests_.ev_request import *

from ..tests.core_options_test import *


def test_mock_request2(capsys):
    response = mock_request("test_url")


    with capsys.disabled():
        if verbose:
            print(response , type(response))
            print(response.text)
            inspect(response)
    assert isinstance(response, requests.models.Response)

def test_report2():
    report_test()




# from ..before_main_ import *

from evdspy.EVDSlocal.initial_setup.setup_folders import setup_folders

# from ..setup_options import *
# from ..setup_options import copy_setup_file
from ..series_format.series_creator import *


@dataclass
class SetupInitial():
    def setup(self):
        setup_folders()
        # create_setup_file()
        # create_series_text_example()

    def create_series_text_ex(self):
        create_series_text_example()

    # def only_copy(self):
    #     copy_setup_file()